#include "stdafx.h"
#include "resource.h"
#include "NeMessenger.h"
#include "MainDlg.h"
#include "NeMessengerDlg.h"
#include "ClientConfig.h"
#include "WorkGroupController.h"
#include "ChatDialogManager.h"
#include "XHyperLink.h"
#include "MenuXP.h"
#include "NoticeController.h"
#include "LayeredDlg.h"
#include "AESCrypto.h"
#include "ConfirmationNumberDlg.h"

IMPLEMENT_DYNAMIC(CMainDlg, CDialog)


CMainDlg::CMainDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMainDlg::IDD, pParent)
	, m_pMessengerMain(NULL)
	, m_bSearch(FALSE)
	, m_bCloseDialog(FALSE)
	, m_strOpenChatSearchext(_T(""))
	, m_pChatAlignHelpLayerDlg(NULL)
	, m_nInfoBtnCount(0)
	, m_bEnableNotice(FALSE)
	, m_bEnableBanner(FALSE)
	, m_recentContacts(&m_favoriteController)
{
	TRACE("%s %d\n", __FUNCTION__, __LINE__);
	InitLegacyBtnPos();
}

CMainDlg::~CMainDlg()
{	
	TRACE("%s %d\n", __FUNCTION__, __LINE__);
	m_chatRoomImageList.DeleteImageList();

	m_bmTopBg.DeleteObject();
	m_bmLegacyBg.DeleteObject();
	m_bmSearchBg.DeleteObject();
	ReleaseChatRoomList();

	SAFE_DELETE(m_pChatAlignHelpLayerDlg);

	CNoticeController::DeleteNotice();
}

void CMainDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	DDX_Control(pDX, IDC_FAVORITE_TREE, m_favoriteController.GetTreeCtrl());
	DDX_Control(pDX, IDC_NOTICE_STATIC, m_staticNotice);
	DDX_Control(pDX, IDC_MAIN_CHAT_MANAGER, m_listChatRoom);
	DDX_Control(pDX, IDC_TILE_BUTTON, m_btnTile);
	DDX_Control(pDX, IDC_BUTTON_FAVORITE, m_btnTabFavorite);
	DDX_Control(pDX, IDC_BUTTON_CHAT_LIST, m_btnTabChatList);
	DDX_Control(pDX, IDC_BUTTON_MAIN_LEGACY_PRE, m_btnLegacyPre);
	DDX_Control(pDX, IDC_BUTTON_MAIN_LEGACY_POST, m_btnLegacyPost);
}

BEGIN_MESSAGE_MAP(CMainDlg, CDialog)	
	ON_MESSAGE(UM_NOTIFY, OnNotify)
	ON_MESSAGE(WM_SENDUSERLIST,						OnAddUser)
	ON_MESSAGE(UM_INFO_ABOUTME_EDIT_NICKNAME,		OnInfoAboutMeEditNickName)

	ON_NOTIFY(TVN_BEGINDRAG, IDC_FAVORITE_TREE, &CMainDlg::OnTvnBegindragFavoriteTree)	
	ON_NOTIFY(TVN_ENDLABELEDIT, IDC_FAVORITE_TREE, &CMainDlg::OnTvnEndlabeleditFavoriteTree)	
	ON_NOTIFY(TVN_GETINFOTIP, IDC_FAVORITE_TREE, &CMainDlg::OnTvnGetInfoTipFavoriteTree)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_MAIN_CHAT_MANAGER, &CMainDlg::OnLvnItemchangedMainChatManager)
	ON_NOTIFY(NM_CLICK, IDC_MAIN_CHAT_MANAGER, &CMainDlg::OnNMClickMainChatManager)
	ON_NOTIFY(NM_RCLICK, IDC_MAIN_CHAT_MANAGER, &CMainDlg::OnNMRClickMainChatManager)

	ON_BN_CLICKED(IDC_TILE_BUTTON, &CMainDlg::OnBnClickedTileButton)
	ON_BN_CLICKED(IDC_BUTTON_FAVORITE, &CMainDlg::OnBnClickedTabFavorite)
	ON_BN_CLICKED(IDC_BUTTON_CHAT_LIST, &CMainDlg::OnBnClickedTabChatList)
	ON_BN_CLICKED(IDC_BUTTON_MAIN_LEGACY_PRE, &CMainDlg::OnBnClickedButtonMainLegacyPre)
	ON_BN_CLICKED(IDC_BUTTON_MAIN_LEGACY_POST, &CMainDlg::OnBnClickedButtonMainLegacyPost)

	ON_COMMAND_RANGE(ID_CHAT_ROOM_OPEN, ID_CHAT_ROOM_ALL_CLOSE, OnMenuChatManager)
	ON_COMMAND_RANGE(WM_USER_ID_LIST, (WM_USER_ID_LIST + 1000), OnMenuFavoriteAdd)

	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_MOVE()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

BEGIN_EVENTSINK_MAP(CMainDlg, CDialog)
END_EVENTSINK_MAP()


void CMainDlg::SetParent(CNeMessengerDlg* pDlg)
{
	m_pMessengerMain = pDlg;
}

GTUserStatusType CMainDlg::GetUserStatusFromUserID(CString strUserID)
{
	GTUserStatusType gtStatus = ustOFFLINE;

	const IGTBuddyUser* pUser = g_pMessengerActor->GetBuddyInfo(strUserID);
	if (pUser)
	{
		gtStatus = pUser->GetStatus();
	}

	return gtStatus;
}

void CMainDlg::InitMyProfile(void)
{
	GT_MY_USER_DATA myUserData;
	g_pMessengerActor->GetMyUserInfo(myUserData);

	CString strMsg = (_T(""));

	m_wndInfoAboutMe.SetAboutMeStatus(myUserData.eUserStatusType);

	strMsg.Format(_T("%s"), myUserData.sMyPage.strUserName);
	m_wndInfoAboutMe.SetName(strMsg);

	strMsg.Format(_T("%s"), myUserData.sMyPage.strTalkName);
	m_wndInfoAboutMe.SetNickName(strMsg);
	
	m_wndInfoAboutMe.SetIPT();

	GET_CONFIG()->SetNickName(myUserData.sMyPage.strTalkName);
}

CString CMainDlg::RegNtalkWndClass(CString strClassNamePrefix)
{
	CString strTmpClassName;
	strTmpClassName.Format(_T("%s_%X%X%X"),
						   strClassNamePrefix,
						   GetTickCount(),
						   GetCurrentProcessId(),
						   GetCurrentThreadId());
	RegisterWindowClass(strTmpClassName);
	return strTmpClassName;
}

void CMainDlg::InitLegacySysCtrl()
{
	CString strTmpClassName = RegNtalkWndClass(_T("Ntalk_InfoItem"));

	if (m_wndInfoItemChat.Create(strTmpClassName, _T("Ntalk_InfoItem_Chat"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0,0,0,0), this, ID_INFOITEM_CHAT))
	{
		m_wndInfoItemChat.SetImgBGPath(_T("New_Info_Item_BG.png"), _T("New_Info_Item_BG_Over.png"), _T("New_Info_Item_BG_Click.png"));
		m_wndInfoItemChat.SetImgBG_LRPath(_T("New_Info_Item_BG_L.png"), _T("New_Info_Item_BG_R.png"));
		m_wndInfoItemChat.SetImgMainPath(_T("New_Info_Item_Chat.png"), _T("New_Info_Item_Chat_Over.png"), _T("New_Info_Item_Chat_Click.png"));
		m_wndInfoItemChat.SetImgSubPath(_T("New_Info_Item_Count_BG.png"));
		m_wndInfoItemChat.SetItemName(LoadStringFromResource(IDS_STRING252));
		m_wndInfoItemChat.SetTooltipText(LoadStringFromResource(IDS_STRING252));
		m_wndInfoItemChat.EnableToolTips();
	}

	if (m_wndInfoItemNote.Create(strTmpClassName, _T("Ntalk_InfoItem_Note"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0,0,0,0), this, ID_INFOITEM_NOTE))
	{
		m_wndInfoItemNote.SetImgBGPath(_T("New_Info_Item_BG.png"), _T("New_Info_Item_BG_Over.png"), _T("New_Info_Item_BG_Click.png"));
		m_wndInfoItemNote.SetImgBG_LRPath(_T("New_Info_Item_BG_L.png"), _T("New_Info_Item_BG_R.png"));
		m_wndInfoItemNote.SetImgMainPath(_T("New_Info_Item_Memo.png"), _T("New_Info_Item_Memo_Over.png"), _T("New_Info_Item_Memo_Click.png"));
		m_wndInfoItemNote.SetImgSubPath(_T("New_Info_Item_Count_BG.png"));
		m_wndInfoItemNote.SetItemName(LoadStringFromResource(IDS_STRING_MAIN_NOTE));
		m_wndInfoItemNote.SetTooltipText(LoadStringFromResource(IDS_STRING_MAIN_NOTE));
		m_wndInfoItemNote.EnableToolTips();
	}

	if (m_wndInfoItemMail.Create(strTmpClassName, _T("Ntalk_InfoItem_Mail"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0,0,0,0), this, ID_INFOITEM_MAIL))
	{
		m_wndInfoItemMail.SetImgBGPath(_T("New_Info_Item_BG.png"), _T("New_Info_Item_BG_Over.png"), _T("New_Info_Item_BG_Click.png"));
		m_wndInfoItemMail.SetImgBG_LRPath(_T("New_Info_Item_BG_L.png"), _T("New_Info_Item_BG_R.png"));
		m_wndInfoItemMail.SetImgMainPath(_T("New_Info_Item_Mail.png"), _T("New_Info_Item_Mail_Over.png"), _T("New_Info_Item_Mail_Click.png"));
		m_wndInfoItemMail.SetImgSubPath(_T("New_Info_Item_Count_BG.png"));
		m_wndInfoItemMail.SetItemName(LoadStringFromResource(IDS_STRING_MAIN_MAIL));
		m_wndInfoItemMail.SetTooltipText(LoadStringFromResource(IDS_STRING_MAIN_MAIL));
		m_wndInfoItemMail.EnableToolTips();
	}

	if (m_wndInfoItemVOC.Create(strTmpClassName, _T("Ntalk_InfoItem_Voc"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0,0,0,0), this, ID_INFOITEM_VOC))
	{
		m_wndInfoItemVOC.SetImgBGPath(_T("New_Info_Item_BG.png"), _T("New_Info_Item_BG_Over.png"), _T("New_Info_Item_BG_Click.png"));
		m_wndInfoItemVOC.SetImgBG_LRPath(_T("New_Info_Item_BG_L.png"), _T("New_Info_Item_BG_R.png"));
		m_wndInfoItemVOC.SetImgMainPath(_T("New_Info_Item_VOC.png"), _T("New_Info_Item_VOC_Over.png"), _T("New_Info_Item_VOC_Click.png"));
		m_wndInfoItemVOC.SetImgSubPath(_T("New_Info_Item_Count_BG.png"));
		m_wndInfoItemVOC.SetItemName(_T("VOC"));
		m_wndInfoItemVOC.SetTooltipText(_T("VOC"));
		m_wndInfoItemVOC.EnableToolTips();
	}

	if (m_wndInfoItemSign.Create(strTmpClassName, _T("Ntalk_InfoItem_Sign"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0,0,0,0), this, ID_INFOITEM_SIGN))
	{
		m_wndInfoItemSign.SetImgBGPath(_T("New_Info_Item_BG.png"), _T("New_Info_Item_BG_Over.png"), _T("New_Info_Item_BG_Click.png"));
		m_wndInfoItemSign.SetImgBG_LRPath(_T("New_Info_Item_BG_L.png"), _T("New_Info_Item_BG_R.png"));
		m_wndInfoItemSign.SetImgMainPath(_T("New_Info_Item_Sign.png"), _T("New_Info_Item_Sign_Over.png"), _T("New_Info_Item_Sign_Click.png"));
		m_wndInfoItemSign.SetImgSubPath(_T("New_Info_Item_Count_BG.png"));
		m_wndInfoItemSign.SetItemName(LoadStringFromResource(IDS_STRING_MAIN_APPROVAL));
		m_wndInfoItemSign.SetTooltipText(LoadStringFromResource(IDS_STRING_MAIN_APPROVAL));
		m_wndInfoItemSign.EnableToolTips();
	}

	if (m_wndInfoItemSurvey.Create(strTmpClassName, _T("Ntalk_InfoItem_Poll"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0,0,0,0), this, ID_INFOITEM_SURVEY))
	{
		m_wndInfoItemSurvey.SetImgBGPath(_T("New_Info_Item_BG.png"), _T("New_Info_Item_BG_Over.png"), _T("New_Info_Item_BG_Click.png"));
		m_wndInfoItemSurvey.SetImgBG_LRPath(_T("New_Info_Item_BG_L.png"), _T("New_Info_Item_BG_R.png"));
		m_wndInfoItemSurvey.SetImgMainPath(_T("New_Info_Item_Survey.png"), _T("New_Info_Item_Survey_Over.png"), _T("New_Info_Item_Survey_Click.png"));
		m_wndInfoItemSurvey.SetImgSubPath(_T("New_Info_Item_Count_BG.png"));
		m_wndInfoItemSurvey.SetItemName(LoadStringFromResource(IDS_STRING_MAIN_SURVEY));
		m_wndInfoItemSurvey.SetTooltipText(LoadStringFromResource(IDS_STRING_MAIN_SURVEY));
		m_wndInfoItemSurvey.EnableToolTips();
	}

	if (m_wndInfoItemWorkOrg.Create(strTmpClassName, _T("Ntalk_InfoItem_WorkOrg"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0,0,0,0), this, ID_INFOITEM_WORKORG))
	{
		m_wndInfoItemWorkOrg.SetImgBGPath(_T("New_Info_Item_BG.png"), _T("New_Info_Item_BG_Over.png"), _T("New_Info_Item_BG_Click.png"));
		m_wndInfoItemWorkOrg.SetImgBG_LRPath(_T("New_Info_Item_BG_L.png"), _T("New_Info_Item_BG_R.png"));
		m_wndInfoItemWorkOrg.SetImgMainPath(_T("New_Info_Item_WorkOrg.png"), _T("New_Info_Item_WorkOrg_Over.png"), _T("New_Info_Item_WorkOrg_Click.png"));		
		m_wndInfoItemWorkOrg.SetItemName(LoadStringFromResource(IDS_STRING_MAIN_ORGCHART));
		m_wndInfoItemWorkOrg.SetTooltipText(LoadStringFromResource(IDS_STRING_MAIN_ORGCHART));
		m_wndInfoItemWorkOrg.EnableToolTips();
	}

	if (m_wndInfoItemIN.Create(strTmpClassName, _T("Ntalk_InfoItem_IN"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0,0,0,0), this, ID_INFOITEM_IN))
	{
		m_wndInfoItemIN.SetImgBGPath(_T("New_Info_Item_BG.png"), _T("New_Info_Item_BG_Over.png"), _T("New_Info_Item_BG_Click.png"));
		m_wndInfoItemIN.SetImgBG_LRPath(_T("New_Info_Item_BG_L.png"), _T("New_Info_Item_BG_R.png"));
		m_wndInfoItemIN.SetImgMainPath(_T("New_Info_Item_IN.png"), _T("New_Info_Item_IN_Over.png"), _T("New_Info_Item_IN_Click.png"));		
		m_wndInfoItemIN.SetItemName(LoadStringFromResource(IDS_STRING378));
		m_wndInfoItemIN.SetTooltipText(LoadStringFromResource(IDS_STRING378));
		m_wndInfoItemIN.EnableToolTips();
	}

	if (m_wndInfoItemSAM.Create(strTmpClassName, _T("Ntalk_InfoItem_SAM"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CRect(0,0,0,0), this, ID_INFOITEM_SAM))
	{
		m_wndInfoItemSAM.SetImgBGPath(_T("New_Info_Item_BG.png"), _T("New_Info_Item_BG_Over.png"), _T("New_Info_Item_BG_Click.png"));
		m_wndInfoItemSAM.SetImgBG_LRPath(_T("New_Info_Item_BG_L.png"), _T("New_Info_Item_BG_R.png"));
		m_wndInfoItemSAM.SetImgMainPath(_T("New_Info_Item_SAM.png"), _T("New_Info_Item_SAM_Over.png"), _T("New_Info_Item_SAM_Click.png"));		
		m_wndInfoItemSAM.SetItemName(LoadStringFromResource(IDS_STRING374));
		m_wndInfoItemSAM.SetTooltipText(LoadStringFromResource(IDS_STRING374));
		m_wndInfoItemSAM.EnableToolTips();
	}
}

void CMainDlg::InitUserVolumnCtrl()
{
	CString strTmpClassName = RegNtalkWndClass(_T("Ntalk_InfoVolume"));

	CRect rtInfoVolume(0, 0, 0, 0);
	if (m_wndInfoVolume.Create(strTmpClassName, _T("Ntalk_Info_Volume"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, rtInfoVolume, this, ID_INFO_VOLUME))
	{
		m_wndInfoVolume.SetImgBGPath(_T("New_Info_Volume_BG.png"));
		m_wndInfoVolume.SetImgVolumePath(_T("New_Info_Volume_Real_B.png"), _T("New_Info_Volume_Real_BG_B.png"));
		m_wndInfoVolume.SetImgVolumePath_R(_T("New_Info_Volume_Real_R.png"), _T("New_Info_Volume_Real_BG_R.png"));
	}
}

void CMainDlg::InitInfoAboutMe()
{
	CString strTmpClassName = RegNtalkWndClass(_T("Ntalk_InfoAboutMe"));

	CRect rtInfoAboutMe(0, 0, 0, 0);
	if (m_wndInfoAboutMe.Create(strTmpClassName, _T("Ntalk_InfoAboutMe"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, rtInfoAboutMe, this, ID_INFO_ABOUTME))
	{
		m_wndInfoAboutMe.SetImgBGPath(_T("New_Info_AboutMe_BG.png"));
		if( g_stNtalkInfo.strCorp == _T("hanatour") )
			m_wndInfoAboutMe.SetImgLogoPath(_T("New_Info_AboutMe_Logo.png"));
		else
			m_wndInfoAboutMe.SetImgLogoPath(_T("New_Info_AboutMe_Logo_blank.png"));//Ŀ NTSΰ!
		m_wndInfoAboutMe.SetImgStatusOverBGPath(_T("New_Info_AboutMe_Status_Over_BG.png"));
		m_wndInfoAboutMe.SetImgIPTOverBGPath(_T("New_Info_ipt_Status_Over_BG.png"));

		CString strEmptyNickName;
		strEmptyNickName.LoadString(IDS_STRING244);
		m_wndInfoAboutMe.SetEmptyNickNameText(strEmptyNickName);
	}

	InitMyProfile();
}

void CMainDlg::InitInfoSearch()
{
	CString strTmpClassName = RegNtalkWndClass(_T("Ntalk_InfoSearch"));

	CRect rtInfoSearch(0, 0, 0, 0);
	if (m_wndSearch.Create(strTmpClassName, _T("Ntalk_InfoSearch"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, rtInfoSearch, this, ID_INFO_SEARCH))
	{
		m_wndSearch.SetImgBGPath(_T("Main_searchform_bg.png"));
		m_wndSearch.SetImgSearchPath(_T("Main_btn_search_Normal.png"), _T("Main_btn_search_Over.png"), _T("Main_btn_search_Click.png"));
		m_wndSearch.SetEditFont(&g_ftDefault9PT);
		//m_wndSearch.SetEditPosTopY(4);
		m_wndSearch.SetEditBkColor(RGB(255, 255, 255));

		m_wndSearch.SetDefaultText(LoadStringFromResource(IDS_STRING245));
		m_wndSearch.SetIntroducingText();
		m_wndSearch.SetSearchText(_T(""));

		m_wndSearch.SetResetSearchText(TRUE);
	}

	//  ̹
	LoadBitmapFromPath(m_bmSearchBg, GetAppImgFile(_T("Main_search_bg.png")));

}

void CMainDlg::InitOutlinkCtrl()
{
	CString strTmpClassName = RegNtalkWndClass(_T("Ntalk_InfoOutlink"));

	CRect rtInfoOutlink(0, 0, 0, 0);
	if (m_wndOutlink.Create(strTmpClassName, _T("Ntalk_InfoOutlink"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, rtInfoOutlink, this, ID_INFO_OUTLINK))
	{
		m_wndOutlink.SetImgBGPath(_T("New_Info_Outlink_BG.png"));
		m_wndOutlink.SetImgItemPath(_T("New_Info_Outlink_Item.png"), _T("New_Info_Outlink_Item_Over.png"), _T("New_Info_Outlink_Item_Click.png"));
		m_wndOutlink.SetImgItemBorderPath(_T("New_Info_Outlink_Border.png"));

		m_wndOutlink.SetFontConfig(g_defaultFontFaceName, DEF_ITEM_FONT_HEIGHT, DEF_ITEM_FONT_STYLE, DEF_ITEM_FONT_COLOR);
	}
}

void CMainDlg::InitBannerCtrl()
{
	CString strTmpClassName = RegNtalkWndClass(_T("Ntalk_InfoBanner"));

	CRect rtInfoBanner(0, 0, 0, 0);
	if (m_wndBanner.Create(strTmpClassName, _T("Ntalk_InfoBanner"), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, rtInfoBanner, this, ID_INFO_BANNER))
	{
		;	// Do nothing...
	}
}

BOOL CMainDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	TRACE("%s %d %d\n", __FUNCTION__, __LINE__, GetSafeHwnd());

	if( g_stNtalkInfo.strCorp == _T("hanatour") )
	{
		m_bEnableBanner = TRUE;
		m_bEnableNotice = TRUE;

		// delete old log files
		CString logPath =  GetAppDataNtalkDir() + _T("\\log");
		DeleteDirectory(logPath);
		CreateDirectory(logPath, NULL);
	}

	// My Information Controls
	InitInfoAboutMe();

	// User Volumn
	InitUserVolumnCtrl();
	m_pMessengerMain->ProcTotalUsedCapa();

	// ƿũ [2012.5.16 yangjae.park@nhn.com]
	m_pMessengerMain->ProcListOutLink();

	// Legacy System
	InitLegacySysCtrl();

	// Search Controls
	InitInfoSearch();

	m_favoriteController.InitView(this);

	InitTabButtons();
	InitLists();

	OnBnClickedTabFavorite();


	// Outlink
	InitOutlinkCtrl();

	// Banner
	if( m_bEnableBanner )
		InitBannerCtrl();

	// Initialize Timers
	SetAllTimer();
	
	__ResourceLoad(TRUE);
	ReSizeCtrl();

	TRACE("%s %d\n", __FUNCTION__, __LINE__);

	return FALSE;  // return TRUE unless you set the focus to a control
	// : OCX Ӽ  FALSE ȯؾ մϴ.
}

void CMainDlg::OnTvnBegindragFavoriteTree(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);	
	*pResult = 0;
}

void CMainDlg::OnTimer(UINT_PTR nIDEvent)
{
	if (nIDEvent == TOTAL_USED_CAPA_TIMER)
	{
		m_pMessengerMain->ProcTotalUsedCapa();
	}
	else if (nIDEvent == NOTICE_LIST_TIMER)
	{
		m_pMessengerMain->ProcListNotice();
	}
	else if (nIDEvent == OUTLINK_LIST_TIMER)
	{
		m_pMessengerMain->ProcListOutLink();
	}
	else if (nIDEvent == REFRESH_TICKET_TIMER)
	{
		// Ƽ  н 1  ٽ õϹǷ, ϴ Ÿ̸Ӹ   缳 Ѵ.
		KillTimer(REFRESH_TICKET_TIMER);
		
		m_pMessengerMain->ProcRefreshTicket();

		SetTimer(REFRESH_TICKET_TIMER, 10 * 59 * 1000, NULL);
	}
	else if(nIDEvent == REFRESH_BANNER_LIST)
	{
		KillTimer(REFRESH_BANNER_LIST);
		g_pMessengerActor->GetWASActor()->RequestListBanner();
	}
	else if(nIDEvent == REFRESH_BUDDY_STATUS)
	{
		std::vector<wstring> vecUsers;
		g_pMessengerActor->RequestGetBuddyStatus(vecUsers);
	}
	else if (nIDEvent == LEGACY_COUNT_TIMER)
	{
		m_pMessengerMain->ProcGetHanatourCount();
	}
	else if(nIDEvent == SHOW_LAYERED_WINDOW_TIMER)
	{
		KillTimer(SHOW_LAYERED_WINDOW_TIMER);
		ShowAlignHelpMsg();	// ȭâ ı ȳâ ش
	}

	CDialog::OnTimer(nIDEvent);
}

void CMainDlg::OnDestroy()
{
	TRACE("%s %d\n", __FUNCTION__, __LINE__);

	KillAllTimer();

	if (m_wndInfoAboutMe.GetSafeHwnd())
		m_wndInfoAboutMe.DestroyWindow();

	if (m_wndInfoItemChat.GetSafeHwnd())	
		m_wndInfoItemChat.DestroyWindow();
	if (m_wndInfoItemNote.GetSafeHwnd())
		m_wndInfoItemNote.DestroyWindow();
	if (m_wndInfoItemMail.GetSafeHwnd())
		m_wndInfoItemMail.DestroyWindow();
	if (m_wndInfoItemVOC.GetSafeHwnd())
		m_wndInfoItemVOC.DestroyWindow();
	if (m_wndInfoItemSign.GetSafeHwnd())
		m_wndInfoItemSign.DestroyWindow();
	if (m_wndInfoItemSurvey.GetSafeHwnd())
		m_wndInfoItemSurvey.DestroyWindow();
	if (m_wndInfoItemWorkOrg.GetSafeHwnd())
		m_wndInfoItemWorkOrg.DestroyWindow();
	if (m_wndInfoItemIN.GetSafeHwnd())
		m_wndInfoItemIN.DestroyWindow();
	if (m_wndInfoItemSAM.GetSafeHwnd())
		m_wndInfoItemSAM.DestroyWindow();
	if (m_wndInfoVolume.GetSafeHwnd())
		m_wndInfoVolume.DestroyWindow();

	if (m_wndSearch.GetSafeHwnd())
		m_wndSearch.DestroyWindow();
	if (m_wndOutlink.GetSafeHwnd())
		m_wndOutlink.DestroyWindow();
	if (m_wndBanner.GetSafeHwnd())
		m_wndBanner.DestroyWindow();

	CDialog::OnDestroy();
}

void CMainDlg::OnSize(UINT nType, int cx, int cy)
{
	CDialog::OnSize(nType, cx, cy);

	::RedrawWindow(GetSafeHwnd(), NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_FRAME);
	ReSizeCtrl();
}

// Ʈѵ ġ
void CMainDlg::ReSizeCtrl(void)
{
	CRect rcClient;
	GetClientRect(rcClient);

	CRect rtAboutMe;

	if (m_wndInfoAboutMe.GetSafeHwnd())
	{
		rtAboutMe = rcClient;
		rtAboutMe.bottom = rtAboutMe.top + m_wndInfoAboutMe.GetImgBGHeight();

		m_wndInfoAboutMe.MoveWindow(&rtAboutMe);
	}

	//  ư ѿ  Show/Hide  ǹǷ ⿡ Ͱ ٰ ǴѴ.
	LONG lLegacyBottom = ResizeLegacyButtons(rcClient, rtAboutMe);

	CRect rtSearch;
	if (m_wndSearch.GetSafeHwnd())
	{
		int nImgWidth = m_wndSearch.GetImgBGWidth();
		int nImgHeight = m_wndSearch.GetImgBGHeight();

		rtSearch = rcClient;
		rtSearch.left += DEF_SEARCH_BAR_GAP_X_L;
		rtSearch.right -= DEF_SEARCH_BAR_GAP_X_R;
		rtSearch.top = lLegacyBottom + DEF_SEARCH_BAR_GAP_Y;
		rtSearch.bottom = rtSearch.top + nImgHeight;

		m_wndSearch.MoveWindow(&rtSearch);
		rtSearch.bottom += DEF_SEARCH_BAR_GAP_Y;
	}

	CRect rtOutlink(0, 0, 0, 0);
	if (m_wndOutlink.GetSafeHwnd())
	{
		rtOutlink.left = rcClient.left;
		rtOutlink.right = rcClient.right;
		rtOutlink.bottom = rcClient.bottom;
		rtOutlink.top = rtOutlink.bottom - DEF_OUTLINK_HEIGHT;
		m_wndOutlink.MoveWindow(rtOutlink);
	}

	LONG lFavoriteTreeBottom = rtOutlink.top;

	CRect rtAD(0, 0, 0, 0);
	if (m_bEnableBanner && m_wndBanner.GetSafeHwnd())
	{
		rtAD.left = rcClient.left;
		rtAD.right = rcClient.right;
		rtAD.bottom = rtOutlink.top;
		rtAD.top = rtAD.bottom - DEF_BANNER_HEIGHT;

		m_wndBanner.MoveWindow(rtAD);
	}

	CRect rtNotice;
	if( m_staticNotice.GetSafeHwnd() )
	{
		if (m_bEnableNotice)
		{
			m_staticNotice.GetWindowRect(&rtNotice);
			int cy = rtNotice.Height();

			rtNotice.left = rcClient.left;
			rtNotice.right = rcClient.right;
			rtNotice.bottom = rtAD.top;
			rtNotice.top = rtNotice.bottom - cy;

			m_staticNotice.MoveWindow(rtNotice);
			m_staticNotice.ShowWindow(SW_SHOW);

			lFavoriteTreeBottom = rtNotice.top;
		}
		else
			m_staticNotice.ShowWindow(SW_HIDE);
	}

	CRect rtTree(0, 0, 0, 0);
	rtTree = rcClient;
	rtTree.top = rtSearch.bottom;
	rtTree.bottom = lFavoriteTreeBottom;

	if (m_btnTabFavorite.GetSafeHwnd())
		rtTree.top += m_btnTabFavorite.GetHeight();

	if (m_favoriteController.GetTreeCtrl().GetSafeHwnd())
		m_favoriteController.MoveView(rtTree);	

	if (m_listChatRoom.GetSafeHwnd())
		m_listChatRoom.MoveWindow(rtTree, TRUE);

	AdjustAlignButtons(rtSearch);
}

void CMainDlg::OnTvnEndlabeleditFavoriteTree(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMTVDISPINFO pTVDispInfo = reinterpret_cast<LPNMTVDISPINFO>(pNMHDR);	
	*pResult = 0;
}

void CMainDlg::StartNoticeList(const std::vector<NE_NOTICE_DATA> &vecNoticeList)
{
	if (TRUE != m_bEnableNotice)
		return;

	if (vecNoticeList.size() <= 0 || m_staticNotice.GetSafeHwnd() == NULL)
		return;

	m_staticNotice.DelTxtFrm();

	m_vecNoticeList.clear();

	m_vecNoticeList = vecNoticeList;

	std::vector<NE_NOTICE_DATA>::const_iterator iter = vecNoticeList.begin();
	for(; iter != vecNoticeList.end(); iter++)
	{
		m_staticNotice.AddTxtFrm((*iter).strTitle.c_str(), _T("M"), 0);
	}

	m_staticNotice.BeginScrolling(this);
}

void CMainDlg::RedrawOutlink()
{
	if (m_wndOutlink.GetSafeHwnd())
	{
		m_wndOutlink.ClearOutlinkItemAll();

		OUTLINK_MAP::iterator iter = GET_CONFIG()->m_mapOutLinkList.begin();
		for( ; iter != GET_CONFIG()->m_mapOutLinkList.end(); ++iter )
		{
			m_wndOutlink.AddOutlinkItem(iter->second);
		}

		m_wndOutlink.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_ALLCHILDREN);
		m_wndOutlink.UpdateBtnInfo();
	}
}

LRESULT CMainDlg::OnNotify(WPARAM wParam, LPARAM lParam)
{
	// wParam : ˸Ÿ, lParam : idx
	// 1:ް. 2:, 3:ܺθũ
	//AfxMessageBox(_T(" ˸Դϴ."));
	int nType = (int)wParam;
	int nIdx = (int)lParam;

	if (nType == 2)
	{
		NE_NOTICE_DATA noticeData = m_vecNoticeList[nIdx];
		CString strUrl; 
		strUrl.Format(NTALK_NOTICE_URL, UrlEncode(g_stNtalkInfo.strCorp), UrlEncode(g_stNtalkInfo.strId), UrlEncode(_T("msn_pc")), UrlEncode((CString)g_pMessengerActor->GetMessengerTicket()), noticeData.nSeq);
		
		CNoticeController *pNotice = CNoticeController::CreateNotice(strUrl);
		pNotice->ShowNotice();

	}
	else if (nType == 3)
	{
		NE_OUTLINK_DATA outlinkData = m_vecOutLinkList[nIdx];
		CNoticeController *pNotice = CNoticeController::CreateNotice(outlinkData.strLinkURL.c_str());
		pNotice->ShowNotice();		
	}

	return 0L;
}


//Notes :  (ģ  ) -> Drag & Drop -> m_ctrlFavoriteTree (ģ ߰) "ProcAddFavorite(int nGroupId, CString strUserId)"
// step 1 : Treectrl  õ  ׷ ̵
// ģ ߰ // "ProcAddFavorite(int nGroupId, CString strUserId)"
LRESULT CMainDlg::OnAddUser(WPARAM wParam, LPARAM lParam)
{
	CStringArray* pArrUser = (CStringArray*)lParam;
	if (!pArrUser)		
		return -1;		
	if (NULL == m_favoriteController.GetTreeCtrl().GetSafeHwnd())
		return -1;

	int nTargetGroupID = m_favoriteController.GetSelectedTargetGroupID();

	if (nTargetGroupID > 0)
	{
		GT_MY_USER_DATA myUserData;
		g_pMessengerActor->GetMyUserInfo(myUserData);	

		vector<wstring> vecUserID;
		for( int i = 0; i < pArrUser->GetSize(); i++)
		{
			if(myUserData.strUserID !=  pArrUser->GetAt(i))
			{
				vecUserID.push_back((LPCWSTR)pArrUser->GetAt(i));
			}
		}

		if( vecUserID.empty() != true )
		{
			m_favoriteController.RequestAddBuddy(nTargetGroupID, vecUserID);
		}
	}

	return 0L;
}


void CMainDlg::OnNMRClickMainChatManager(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMITEMACTIVATE pNMItemActivate = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	CPoint point(pNMItemActivate->ptAction);

	UINT uFlags;
	int nItem = m_listChatRoom.HitTest(point, &uFlags);

	if (!(uFlags & LVHT_ONITEM))
		return;

 	CPoint pt;
 	GetCursorPos(&pt);

	CMenu menuList;
	menuList.LoadMenu(IDR_MEMU_COMMON_MENU);

	int nSubMenu = 6;

	if (m_listChatRoom.GetSelectedCount() == 1)
		MakeFavoriteMenu(&menuList, nSubMenu);	// ȭ ߰ Ʈ  

	menuList.GetSubMenu(nSubMenu)->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, this);

	*pResult = 0;
}

void CMainDlg::GetSelectedUserIDList(CStringList &selectedIdList)
{
	POSITION pos = m_listChatRoom.GetFirstSelectedItemPosition();

	while (pos)
	{
		int nIndex = m_listChatRoom.GetNextSelectedItem(pos);
		int nRoomID = (int)m_listChatRoom.GetItemData(nIndex);

		CChatDlg *pDlg = GET_APP()->m_pChatDialogManager->FindCreatedSessionChatDlg(nRoomID);

		if (!pDlg)
			continue;

		int nUserCnt = pDlg->GetChatUserMapCount();
		CString strID;

		for (int i = 0; i < nUserCnt; i++)
		{
			strID = pDlg->GetChatUserID(i);

			if (pDlg->GetMyUserID().CompareNoCase(strID) != 0)
				selectedIdList.AddTail(strID);
		}
	}
}

void CMainDlg::MakeFavoriteMenu(CMenu* menu, int nSub)
{
	// ã ׷ Ʈ( )
	CMenu menuFavoriteList;

	if (!menuFavoriteList.CreatePopupMenu())
		return;

	// ⺻ ޴(ã Ʈ  θ ޴)
	CMenu* pPopup = menu->GetSubMenu(nSub);

	if (!pPopup)
		return;

	m_GroupList.clear();

	size_t nGroupCount = g_pMessengerActor->GetBuddyGroupCount();
	int nAddableBuddyGroupCount = 0;	// ģ߰  ã ׷ 
	// ׷ ȿ ڰ ϵǾ   츸 .
	int nMenuIndex = 0;

	if (nGroupCount > 0)
	{
		CStringList selectedIdList;
		GetSelectedUserIDList(selectedIdList);	// õ ID Ʈ

		// ã ׷
		for (size_t i = 0; i < nGroupCount; i++)
		{
			const IGTBuddyGroup *pGroup = g_pMessengerActor->GetBuddyGroupInfoFromIndex(i);

			CString strGroupName = pGroup->GetName();
			int nGroupID = pGroup->GetID();

			// ü  Ʈ ͼ
			// Ѹ̶ ϵ  ׷ ߰
			if (!IsAllIdsInBuddyGroup(selectedIdList, nGroupID))	// õ  Ʈ  ã ׷ ȿ   Ѹ̶  
			{
				m_GroupList.insert(std::make_pair(WM_USER_ID_LIST + nMenuIndex, nGroupID));
				menuFavoriteList.AppendMenu(MF_STRING, WM_USER_ID_LIST + nMenuIndex++, strGroupName);
				nAddableBuddyGroupCount++;
			}
		}
	}

	pPopup->InsertMenu(0, MF_POPUP | MF_BYPOSITION, (UINT_PTR)menuFavoriteList.m_hMenu, LoadStringFromResource(IDS_STRING262));
}

BOOL CMainDlg::IsAllIdsInBuddyGroup(CStringList& selectedIdList, int nGroupID)
{
	BOOL bAllUsersAreInTheBuddyGroup = TRUE;	// selectedIdList  ڰ nGroupID ȿ  ִٸ TRUE

	int buddyCount = g_pMessengerActor->GetBuddyCount(nGroupID);

	CStringList buddyIdInGroup;
	for (int index = 0; index < buddyCount; index++)
	{
		const IGTBuddyUser* pUser = g_pMessengerActor->GetBuddyInfoFromIndex(nGroupID, index);
		CString strBuddyID = pUser->GetUserID();
		buddyIdInGroup.AddTail(strBuddyID);
	}

	POSITION pos = selectedIdList.GetHeadPosition();

	while (pos)
	{
		CString strID = selectedIdList.GetNext(pos);

		if (buddyIdInGroup.Find(strID) == NULL)
		{
			bAllUsersAreInTheBuddyGroup = FALSE;
			break;
		}
	}

	return bAllUsersAreInTheBuddyGroup;
}

void CMainDlg::OnTvnGetInfoTipFavoriteTree(NMHDR *pNMHDR, LRESULT *pResult)
{
	/*	LPNMTVGETINFOTIP pGetInfoTip = reinterpret_cast<LPNMTVGETINFOTIP>(pNMHDR);
	strSelectedText2 =  m_ctrlFavoriteTree.GetItemText(pGetInfoTip->hItem);				
	int strLen = 0;
	strLen = strSelectedText2.GetLength();		
	pGetInfoTip->pszText = (LPWSTR)(LPCWSTR)strSelectedText2;
	pGetInfoTip->cchTextMax = strLen;		
	*/
	*pResult = 0;
}

BOOL CMainDlg::OnCommand(WPARAM wParam, LPARAM lParam)
{
	int nID = LOWORD(wParam);
	switch (nID)
	{
	case ID_INFOITEM_CHAT:
		{
			DoInfoItemChat();
		}
		break;
	case ID_INFOITEM_NOTE:
		{
			DoInfoItemNote();
		}
		break;
	case ID_INFOITEM_MAIL:
		{
			DoInfoItemMail();
		}
		break;
	case ID_INFOITEM_VOC:
		{
			DoInfoItemVOC();
		}
		break;
	case ID_INFOITEM_SIGN:
		{
			DoInfoItemSign();
		}
		break;
	case ID_INFOITEM_SURVEY:
		{
			DoInfoItemSurvey();
		}
		break;
	case ID_INFOITEM_IN:
		{
			DoInfoItemIN();
		}
		break;
	case ID_INFOITEM_SAM:
		{
			DoInfoItemSAM();
		}
		break;
	case ID_INFOITEM_WORKORG:
		{
			GET_MESSENGER_DLG()->LoadOrganization(SHOW_ORG, this);
		}
		break;
	case ID_INFO_ABOUTME:
		{
			CRect rtAboutMe;
			m_wndInfoAboutMe.GetWindowRect(&rtAboutMe);

			CRect rtStatus;
			rtStatus = m_wndInfoAboutMe.GetStatusRect();

			CPoint pt;
			pt.x = rtAboutMe.left + rtStatus.left;
			pt.y = rtAboutMe.top + rtStatus.bottom;

			LoadAboutMeStatusPopup(pt);
		}
		break;
	case ID_INFO_SEARCH:
		{
			CString strSearchText = m_wndSearch.GetSearchText();
			DoInfoSearch(strSearchText);
		}
		break;
	case ID_IPT_ABOUTME:
		{
			CRect rtAboutMe;
			m_wndInfoAboutMe.GetWindowRect(&rtAboutMe);

			CRect rtIPTStatus;
			rtIPTStatus = m_wndInfoAboutMe.GetIPTStatusRect();

			CPoint pt;
			pt.x = rtAboutMe.left + rtIPTStatus.left;
			pt.y = rtAboutMe.top + rtIPTStatus.bottom;

			LoadAboutIPTStatusPopup(pt);
		}	
	}

	return CDialog::OnCommand(wParam, lParam);
}

void CMainDlg::SetInfoItemCount(NTHanatourCountType eType, int nCount, CString strURL)
{
	switch (eType)
	{
	case hanatourVoc:
		{
			m_wndInfoItemVOC.SetItemCount(nCount);
			m_wndInfoItemVOC.SetItemURL(strURL);
		}
		break;
	case hanatourSign:
		{
			m_wndInfoItemSign.SetItemCount(nCount);
			m_wndInfoItemSign.SetItemURL(strURL);
		}
		break;
	case hanatourPoll:
		{
			m_wndInfoItemSurvey.SetItemCount(nCount);
			m_wndInfoItemSurvey.SetItemURL(strURL);
		}
	case hanatourIN:
		{
			m_wndInfoItemIN.SetItemCount(0);
			m_wndInfoItemIN.SetItemURL(strURL);
		}
	case hanatourSAM:
		{
			m_wndInfoItemSAM.SetItemCount(0);
			m_wndInfoItemSAM.SetItemURL(strURL);
		}
		break;
	}
}

void CMainDlg::DoInfoItemChat()
{
	CString strTicket = CString(g_pMessengerActor->GetMessengerTicket());
	CString strLang = GetLanguage();

	CString strUrl = _T("");
	strUrl.Format(NTALK_CHAT_URL, strTicket, strLang);

	if (TRUE == CXHyperLink::UseAlreadyOpenedIE(_T("nnote.ncloudbiz.com/talk")))
		return;

	CXHyperLink::GotoURL(strUrl, SW_SHOWNORMAL);
}

void CMainDlg::DoInfoItemNote()
{
	CString strTicket = CString(g_pMessengerActor->GetMessengerTicket());
	CString strLang = GetLanguage();

	CString strUrl = _T("");
	strUrl.Format(NTALK_MESSAGE_URL, strTicket, strLang);

	if (TRUE == CXHyperLink::UseAlreadyOpenedIE(_T("nnote.ncloudbiz.com"), _T("talk")))
		return;

	CXHyperLink::GotoURL(strUrl, SW_SHOWNORMAL);
}

void CMainDlg::DoInfoItemMail()
{
	if (TRUE == CXHyperLink::UseAlreadyOpenedIE(_T("nmail.ncloudbiz.com")))
		return;

	CString strTicket = CString(g_pMessengerActor->GetMessengerTicket());
	CString strUrl;
	strUrl.Format(NTALK_MAIL_URL, strTicket);
	CXHyperLink::GotoURL(strUrl, SW_SHOWNORMAL);
}

void CMainDlg::DoInfoItemVOC()
{
	if (m_wndInfoItemVOC.GetItemURL().IsEmpty())
		return;

	CString strLoginParam = MakeHanatourLoginData(_T("VOC"));
	if (strLoginParam.IsEmpty())
		return;

	CString strURL;
	strURL.Format(_T("%s?whoAmI=H&loginType=S&crm=Y&crmTabMenuInit=Y&%s&crmRedirectValue=06"), m_wndInfoItemVOC.GetItemURL(), strLoginParam);

	OpenURLService(strURL);
}

void CMainDlg::DoInfoItemSurvey()
{
	if (m_wndInfoItemSurvey.GetItemURL().IsEmpty())
		return;

	CString strLoginParam = MakeHanatourLoginData(LoadStringFromResource(IDS_STRING_MAIN_SURVEY));
	if (strLoginParam.IsEmpty())
		return;

	CString strURL;
	strURL.Format(_T("%s?whoAmI=H&loginType=S&crm=Y&crmTabMenuInit=Y&%s&crmRedirectValue=33"), m_wndInfoItemSurvey.GetItemURL(), strLoginParam);

	OpenURLService(strURL);
}

CString CMainDlg::GetMyUserID_Base64()
{
	return GetUserID_Base64(g_stNtalkInfo.strId);
}

CString CMainDlg::GetUserID_Base64(CString strUserID)
{
	USES_CONVERSION;
	CAESCrypto crypto;

	char *pszAnsiID = W2A(strUserID);

	if (!pszAnsiID)
		return _T("");

	char szBase64ID[128] = {0,};
	int nB64Len = 0;

	crypto.Base64Encode((const BYTE*)pszAnsiID, strlen(pszAnsiID), szBase64ID, &nB64Len, BASE64_FLAG_NOCRLF);

	return CString(szBase64ID);
}

CString CMainDlg::StringToHex(char* pData)
{
	CString strToHex = _T("");
	int nLen = strlen(pData);
	char szHex[3] = {0,};

	for (int i = 0; i < nLen; i++)
	{
		sprintf_s(szHex, 3, "%02X", pData[i]);
		strToHex += szHex;
	}

	return strToHex;
}

CString CMainDlg::MakeHanatourLoginData(CString strLegacySystemName)
{
	CConfirmationNumberDlg cfDlg(this, strLegacySystemName);
	if (cfDlg.DoModal() != IDOK)
		return _T("");

	const unsigned int lenSHA256 = 32;

	USES_CONVERSION;
	CAESCrypto crypto;

	char *pszAnsiID = W2A(g_stNtalkInfo.strId);
	char *pszAnsiPW = W2A(g_stNtalkInfo.strPassword);
	char *pszAnsiNo = W2A(cfDlg.GetConfirmNo());

	if (!pszAnsiID || !pszAnsiPW || !pszAnsiNo)
		return _T("");

	char szSHA256PW[lenSHA256] = {0,};
	DWORD dwBufSize = lenSHA256;

/*
	// crypto.GetSHA256Hash() Windows XP SP3   ϴ Լ Ͽ ٸ Լ ü
	if (crypto.GetSHA256Hash(pszAnsiPW, strlen(pszAnsiPW), (BYTE*)szSHA256PW, &dwBufSize) == FALSE)
		return _T("");
*/

	if (crypto.GetSHA256(pszAnsiPW, strlen(pszAnsiPW), (BYTE*)szSHA256PW) == FALSE)
		return _T("");

	char szBase64ID[128] = {0,};
	char szBase64PW[128] = {0,};
	char szBase64PW2[256] = {0,};	// PW   (SHA256 -> Base64encoding -> Base64encoding -> 16 ȯ)
	char szBase64No[128] = {0,};
	int nB64Len = 0;

	crypto.Base64Encode((const BYTE*)pszAnsiID, strlen(pszAnsiID), szBase64ID, &nB64Len, BASE64_FLAG_NOCRLF);
	crypto.Base64Encode((const BYTE*)szSHA256PW, lenSHA256, szBase64PW, &nB64Len, BASE64_FLAG_NOCRLF);
	crypto.Base64Encode((const BYTE*)szBase64PW, strlen(szBase64PW), szBase64PW2, &nB64Len, BASE64_FLAG_NOCRLF);
	crypto.Base64Encode((const BYTE*)pszAnsiNo, strlen(pszAnsiNo), szBase64No, &nB64Len, BASE64_FLAG_NOCRLF);

	CString strParam;
	strParam.Format(_T("HanaId=%s&HanaPassword=%s&HanaPassword2=%s"), StringToHex(szBase64ID), StringToHex(szBase64PW2), StringToHex(szBase64No));

	return strParam;
}

void CMainDlg::DoInfoItemSign()
{
	//m_wndInfoItemSign.SetItemURL(_T("http://intra.hanatour.co.kr/edin/login.sso"));
	if (m_wndInfoItemSign.GetItemURL().IsEmpty())
		return;

	CString strLoginParam = MakeHanatourLoginData(LoadStringFromResource(IDS_STRING_MAIN_APPROVAL));

	if (strLoginParam.IsEmpty())
		return;

	CString strURL;
	strURL.Format(_T("%s?whoAmI=H&loginType=S&crm=Y&crmTabMenuInit=Y&%s&crmRedirectValue=10"), m_wndInfoItemSign.GetItemURL(), strLoginParam);

	OpenURLService(strURL);
}

void CMainDlg::DoInfoItemIN()
{
 	if (m_wndInfoItemIN.GetItemURL().IsEmpty())
 		return;

	CString strLoginParam = MakeHanatourLoginData(LoadStringFromResource(IDS_STRING378));
	if (strLoginParam.IsEmpty())
		return;

	CString strURL;
	strURL.Format(_T("%s?whoAmI=H&loginType=S&crm=Y&crmTabMenuInit=Y&%s&crmRedirectValue=28"), m_wndInfoItemIN.GetItemURL(), strLoginParam);

	OpenURLService(strURL);
}

void CMainDlg::DoInfoItemSAM()
{
	if (m_wndInfoItemSAM.GetItemURL().IsEmpty())
		return;

	CString strLoginParam = MakeHanatourLoginData(LoadStringFromResource(IDS_STRING374));
	if (strLoginParam.IsEmpty())
		return;

	CString strURL;
	strURL.Format(_T("%s?whoAmI=H&loginType=S&crm=Y&crmTabMenuInit=Y&%s"), m_wndInfoItemSAM.GetItemURL(), strLoginParam);

	OpenURLService(strURL);
}

void CMainDlg::DoInfoSearch(CString strSearchText)
{
	strSearchText = strSearchText.Trim(_T("% "));	// յ %  ڴ Ѵ.

	if (m_btnTabFavorite.GetSelected())	// ȭ Ͽ ˻( ˻)
	{
		if (strSearchText.GetLength() < 2)
		{	
			NMessageBox(this, LoadStringFromResource(IDS_STRING169));
			m_wndSearch.SetEditSearch(TRUE);

			return;
		}

		GET_MESSENGER_DLG()->GetWorkGroupController()->CreateSearchResultView(strSearchText);
	}
	else	// ȭ Ͽ ˻
	{
		// ȭ Ͽ ˻   ü ȭ Ʈ  ϹǷ ˻ ̸  ʴ´.
		m_bSearch = FALSE;
		
		// ȭ  ʱȭ
		ClearChatRoomList();
		if(strSearchText.IsEmpty())
		{
		   m_listChatRoom.ShowDefaultText(TRUE);
		}
		else
		{
		   m_listChatRoom.ShowEmptyText(TRUE);
		}
		
		GET_APP()->m_pChatDialogManager->InitChatRoomList();

		if (strSearchText.IsEmpty())
		{
			m_wndSearch.SetEditSearch(TRUE);
			return;
		}

		m_bSearch = TRUE;
		m_wndSearch.SetResetSearchText(FALSE);
		m_strOpenChatSearchext = strSearchText;
		InvalidateChatListSearch();
		GET_APP()->m_pChatDialogManager->DisplaySearchChatRoomList(strSearchText);
	}
}

void CMainDlg::ClearFavoriteList()
{
	if (NULL != m_favoriteController.GetTreeCtrl().GetSafeHwnd())
		m_favoriteController.ClearFavoriteList();
}

BOOL CMainDlg::MakeAboutMeSatusPopupMenu(CMenu* pMenu)
{
	CMenuXP* pXpMenu = dynamic_cast<CMenuXP*>(pMenu);
	ASSERT(pXpMenu);
	if (pXpMenu)
	{
		pXpMenu->CreatePopupMenu();
		pXpMenu->SetBackColor(RGB(240, 240, 240));
		pXpMenu->SetSelectedBarColor(RGB(51, 153, 254));	

		CString strRes;
		strRes.LoadString(IDS_STRING131);
		pXpMenu->AppendODMenu(0, new CMenuXPText(1, strRes, AfxGetApp()->LoadIcon(IDI_IC_ONLINE)));
		strRes.LoadString(IDS_STRING132);
		pXpMenu->AppendODMenu(0, new CMenuXPText(2, strRes, AfxGetApp()->LoadIcon(IDI_IC_AFK)));
		strRes.LoadString(IDS_STRING133);
		pXpMenu->AppendODMenu(0, new CMenuXPText(3, strRes, AfxGetApp()->LoadIcon(IDI_IC_OTHERBUSY)));
		strRes.LoadString(IDS_STRING134);
		pXpMenu->AppendODMenu(0, new CMenuXPText(4, strRes, AfxGetApp()->LoadIcon(IDI_IC_STEPOUT)));
		//strRes.LoadString(IDS_STRING135);
		//pMenu->AppendODMenu(0, new CMenuXPText(5, strRes, AfxGetApp()->LoadIcon(IDI_IC_TELBUSY)));
		strRes.LoadString(IDS_STRING136);
		pXpMenu->AppendODMenu(0, new CMenuXPText(5, strRes, AfxGetApp()->LoadIcon(IDI_IC_EATBUSY)));
		strRes.LoadString(IDS_STRING137);
		pXpMenu->AppendODMenu(0, new CMenuXPText(6, strRes, AfxGetApp()->LoadIcon(IDI_IC_MEETBUSY)));
		//strRes.LoadString(IDS_STRING138);
		//pMenu->AppendODMenu(0, new CMenuXPText(8, strRes, AfxGetApp()->LoadIcon(IDI_IC_OFFLINE)));
	}

	return TRUE;
}


BOOL CMainDlg::MakeAboutIPTSatusPopupMenu( CMenu* pMenu )
{
	CMenuXP* pXpMenu = dynamic_cast<CMenuXP*>(pMenu);
	ASSERT(pXpMenu);
	if (pXpMenu)
	{
		pXpMenu->CreatePopupMenu();
		pXpMenu->SetBackColor(RGB(240, 240, 240));
		pXpMenu->SetSelectedBarColor(RGB(51, 153, 254));	

		CString strRes;
		strRes = LoadStringFromResource(IDS_STRING671);
		pXpMenu->AppendODMenu(0, new CMenuXPText(1, strRes, AfxGetApp()->LoadIcon(IDI_ICON_IPT_IDLE)));
		strRes = LoadStringFromResource(IDS_STRING672);
		pXpMenu->AppendODMenu(0, new CMenuXPText(2, strRes, AfxGetApp()->LoadIcon(IDI_ICON_IPT_DND)));
		strRes = LoadStringFromResource(IDS_STRING673);
		pXpMenu->AppendODMenu(0, new CMenuXPText(3, strRes, AfxGetApp()->LoadIcon(IDI_ICON_IPT_FWD)));
		strRes = LoadStringFromResource(IDS_STRING674);
		pXpMenu->AppendODMenu(0, new CMenuXPText(4, strRes, AfxGetApp()->LoadIcon(IDI_ICON_IPT_PAUSE)));
	}

	return TRUE;
}

void CMainDlg::LoadAboutMeStatusPopup(CPoint pt)
{
	CMenuXP* pMenu = new CMenuXP;
	if (!pMenu)
		return;

	MakeAboutMeSatusPopupMenu(pMenu);

	int nCmdID = pMenu->TrackPopupMenu(TPM_LEFTBUTTON | TPM_RETURNCMD, pt.x, pt.y, this);

	delete pMenu;

	if ((GTUserStatusType)nCmdID < ustONLINE || (GTUserStatusType)nCmdID > ustAUTOAWAY)
		return;

	m_pMessengerMain->ProcChangeMyStatus((GTUserStatusType)(nCmdID));
}


void CMainDlg::LoadAboutIPTStatusPopup( CPoint pt )
{
	CMenuXP* pMenu = new CMenuXP;
	if (!pMenu)
		return;

	MakeAboutIPTSatusPopupMenu(pMenu);

	int nCmdID = pMenu->TrackPopupMenu(TPM_LEFTBUTTON | TPM_RETURNCMD, pt.x, pt.y, this);

	delete pMenu;
	if (1 == nCmdID)
	{
		if (GET_MESSENGER_DLG()->GetDND())
		{
			GET_MESSENGER_DLG()->SetDND();
		}

		if (GET_MESSENGER_DLG()->GetForwardingStatus())
			g_pMessengerActor->RequestCallForwarding( g_pMessengerActor->GetUserID(), 0, _T("") );


	}
	else if (2 == nCmdID)
	{
		if (!GET_MESSENGER_DLG()->GetDND())
		{
			GET_MESSENGER_DLG()->SetDND();
		}
		else
			g_pMessengerActor->RequestSetDND( g_pMessengerActor->GetUserID(), 1);
		
	}
	else if(3 == nCmdID)
	{
		if(!GET_MESSENGER_DLG()->GetForwardingStatus())
		{
			GET_MESSENGER_DLG()->OnTopMenuForwarding();
		}
	}
	else if (4 == nCmdID)
	{
		GET_MESSENGER_DLG()->OnTopMenuHoldCall();
	}
}


void CMainDlg::UpdateInfoAboutMeStatus(GTUserStatusType gtStatus)
{
	if (!m_wndInfoAboutMe.GetSafeHwnd())
		return;

	m_wndInfoAboutMe.SetAboutMeStatus(gtStatus);

}

void CMainDlg::UpdateInfoAboutIPTStatus()
{
	if (!m_wndInfoAboutMe.GetSafeHwnd())
		return;

	NTIPTStatusType iptStatus = g_pMessengerActor->GetMyIptStatus();
	if(GET_MESSENGER_DLG()->GetForwardingStatus())
		iptStatus = iptstFWD;
	if (iptStatus == iptstDND)
		GET_MESSENGER_DLG()->SetDND(1);
	m_wndInfoAboutMe.SetAboutIPTStatus(iptStatus);
}


void CMainDlg::UpdateInfoAboutMeNickName(CString strNickName)
{
	//GT_FLOG(L"NickName = %s \n",strNickName);

	if (!m_wndInfoAboutMe.GetSafeHwnd())
		return;

	m_wndInfoAboutMe.SetNickName(strNickName);
}

void CMainDlg::UpdateInfoVolume(UINT nTotalUsedCapa, UINT nLimitedCapa)
{
	if (!m_wndInfoVolume.GetSafeHwnd())
		return;

	g_stNtalkInfo.nCapacity = nLimitedCapa - nTotalUsedCapa;

	m_wndInfoVolume.SetVolume(nTotalUsedCapa);
	m_wndInfoVolume.SetVolumeLimit(nLimitedCapa);
}

LRESULT CMainDlg::OnInfoAboutMeEditNickName(WPARAM wParam, LPARAM lParam)
{
	m_pMessengerMain->ProcRenameTalkname(m_wndInfoAboutMe.GetNickName());
	return 0L;
}

BOOL CMainDlg::PreTranslateMessage(MSG* pMsg)
{
	if (pMsg->message == WM_KEYDOWN)
	{
		switch (pMsg->wParam)
		{
		case VK_ESCAPE:
		case VK_RETURN:
			return TRUE;
		}

		BOOL bShift = ((GetKeyState(VK_SHIFT) & 0x8000) != 0);
		BOOL bControl = ((GetKeyState(VK_CONTROL) & 0x8000) != 0);
		BOOL bAlt = ((GetKeyState(VK_MENU) & 0x8000) != 0);

		if(bControl && !bShift && !bAlt)
		{
			if(pMsg->wParam == 'n' || pMsg->wParam == 'N')
			{
				TRACE("CTRL+N Detected [Open Memo]\n");
				SendMessage(WM_COMMAND, (WPARAM)ID_INFOITEM_NOTE, (LPARAM)0);
			}
		}
	}

	return CDialog::PreTranslateMessage(pMsg);
}

void CMainDlg::KillAllTimer()
{
	KillTimer(TOTAL_USED_CAPA_TIMER);
	KillTimer(REFRESH_TICKET_TIMER);
	KillTimer(REFRESH_BUDDY_STATUS);
	KillTimer(LEGACY_COUNT_TIMER);

	KillTimer(NOTICE_LIST_TIMER);
	KillTimer(REFRESH_BANNER_LIST);
}

void CMainDlg::SetAllTimer()
{
	SetTimer(TOTAL_USED_CAPA_TIMER, 60 * 60 * 1000, NULL);	
	SetTimer(REFRESH_TICKET_TIMER, 10 * 59 * 1000, NULL);	//   ֱ  (5 -> 2ð) ٸ ۰ ġ  ּȭ ϱ  Ϻη 60 ƴ 59 .
	SetTimer(REFRESH_BUDDY_STATUS, 3 * 60 * 1000, NULL);	// ¶  ȭ ֱ 3
	SetTimer(LEGACY_COUNT_TIMER, 5 * 60 * 1000, NULL);
	//SetTimer(SHOW_LAYERED_WINDOW_TIMER, 2 * 1000, NULL);	//  5  1ȸ

	if( m_bEnableNotice )
		SetTimer(NOTICE_LIST_TIMER, 10 * 60 * 1000, NULL);

	if( m_bEnableBanner )
		SetTimer(REFRESH_BANNER_LIST, 1000, NULL);				//  1 ,  ٿ 10 ֱ
}

void CMainDlg::OpenURLService(CString strURL)
{
	if (!strURL.IsEmpty())
	{
/*
		// / µ ϳ/ϳ/  ʴ  ߻ϴ 찡 ִ.
		// Ȥ ̹ ִ â ã κп    ־,  κ ּóѴ.
		if (TRUE == CXHyperLink::UseAlreadyOpenedIE(strURL))
			return;
*/

		CXHyperLink::GotoURL(strURL, SW_SHOWNORMAL);
	}
}

CString CMainDlg::GetLanguage()
{
	CString strLang = _T("");

	if (g_stNtalkInfo.nLang == 0)
		strLang = _T("ko_KR");
	else if (g_stNtalkInfo.nLang == 1)
		strLang = _T("en_US");
	else if (g_stNtalkInfo.nLang == 2)
		strLang = _T("ja_JP");
	else if (g_stNtalkInfo.nLang == 3)
		strLang = _T("zh_CN");
	else if (g_stNtalkInfo.nLang == 4)
		strLang = _T("zh_TW");

	return strLang;
}

void CMainDlg::InitList(CChatListCtrl &list, CStringArray &strHeaderTitle)
{
	LV_COLUMN lvcolumn;
	memset(&lvcolumn, 0, sizeof(lvcolumn));
	lvcolumn.mask = LVCF_FMT | LVCF_SUBITEM | LVCF_TEXT | LVCF_MINWIDTH;

	for (int i = 0; i < strHeaderTitle.GetCount(); i++)
	{
		lvcolumn.fmt = LVCFMT_LEFT;
		lvcolumn.pszText = (LPTSTR)(LPCTSTR)strHeaderTitle.GetAt(i);
		lvcolumn.iSubItem = i;
		lvcolumn.cxMin = 0;
		list.InsertColumn(i, &lvcolumn);
	}

	list.ModifyStyle(0, LVS_OWNERDRAWFIXED, 0);
	list.SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_DOUBLEBUFFER);
	list.SetRowHeight(40);				// Ʈ   
	list.SetNameFont(&g_ftDefault10PTB);	//   Ʈ
	list.SetChatFont(&g_ftDefault9PT);	//  ȭ  Ʈ
}

void CMainDlg::InitLists()
{
	if (NULL != m_chatRoomImageList.GetSafeHandle())
		m_chatRoomImageList.DeleteImageList();

	if (m_chatRoomImageList.Create(28, 28, ILC_COLOR32 | ILC_COLOR | ILC_MASK, 2, 2))
	{
		COLORREF crMask = RGB(0, 0, 0);

		CBitmap bmStatus[2];

		LoadBitmapFromPath(bmStatus[0], GetAppImgFile(_T("Chat_One.png")));
		LoadBitmapFromPath(bmStatus[1], GetAppImgFile(_T("Chat_Group.png")));

		for (int i = 0; i < 2; i++)
		{
			m_chatRoomImageList.Add(&bmStatus[i], crMask);
			bmStatus[i].DeleteObject();
		}
	}

	m_listChatRoom.SetImageList(&m_chatRoomImageList, LVSIL_SMALL);

	const int colWidthChatList[] = {
		50,		// 
		180,	// ȭ 
		90,		//  ȭ ð
		0,		//  ȭ ()
		-1,
	};

	CStringArray arChatRoomListHeaderTitle;
	arChatRoomListHeaderTitle.Add(LoadStringFromResource(IDS_STRING425));	//  
	arChatRoomListHeaderTitle.Add(LoadStringFromResource(IDS_STRING443));	// ȭ 
	arChatRoomListHeaderTitle.Add(LoadStringFromResource(IDS_CHAT_ROOM_START_TIME));	//  ȭ ð
	arChatRoomListHeaderTitle.Add(_T(""));

	InitList(m_listChatRoom, arChatRoomListHeaderTitle);
	m_listChatRoom.ShowDefaultText(TRUE);
	ResizeListColumn(m_listChatRoom, colWidthChatList);
	m_listChatRoom.SetImageColumn(0);
	GET_APP()->m_pChatDialogManager->InitChatRoomList();
}

void CMainDlg::ResizeListColumn(CChatListCtrl &list, const int *arrWidth)
{
	for (int i = 0; arrWidth[i] != -1; i++)
	{
		list.SetColumnWidth(i, arrWidth[i]);
	}

	list.RedrawWindow();
}

void CMainDlg::InitTabButtons()
{
	//  , ׷  ư
	m_btnTabFavorite.SetTabBitmapFile(GetAppImgFile(_T("Main_tab_01.bmp")),RGB(0,0,0));
	m_btnTabFavorite.SetFont(&g_ftDefault9PT);
	m_btnTabFavorite.SetTextColor(RGB(78, 157, 199), RGB(34, 34, 34));
	m_btnTabFavorite.SetButtonText(LoadStringFromResource(IDS_STRING_CHAT_LIST));
	m_btnTabFavorite.ShowWindow(SW_SHOW);

	m_btnTabChatList.SetTabBitmapFile(GetAppImgFile(_T("Main_tab_02.bmp")),RGB(0,0,0));
	m_btnTabChatList.SetFont(&g_ftDefault9PT);
	m_btnTabChatList.SetTextColor(RGB(78, 157, 199), RGB(34, 34, 34));
	m_btnTabChatList.SetButtonText(LoadStringFromResource(IDS_STRING_CHAT_ATTENDING_LIST));
	m_btnTabChatList.ShowWindow(SW_SHOW);

	//  ư  ̹
	LoadBitmapFromPath(m_bmTopBg, GetAppImgFile(_T("Main_tab_bg.png")));

	// Ⱓ ư  ̹
	LoadBitmapFromPath(m_bmLegacyBg, GetAppImgFile(_T("Main_legacy_bg.png")));
}

void CMainDlg::AdjustAlignButtons( const CRect &rtSearch )
{
	int nGap = 4;
	CRect rtButton;

	if (m_btnTile.GetSafeHwnd())
	{
		m_btnTile.GetWindowRect(&rtButton);

		rtButton.left = rtSearch.right + nGap;
		rtButton.right = rtButton.left + m_btnTile.GetImgWidth();
		rtButton.top = rtSearch.top;
		rtButton.bottom = rtButton.top + m_btnTile.GetImgHeight();

		m_btnTile.MoveWindow(rtButton);
		m_btnTile.Invalidate();
	}

	if (m_btnTabFavorite.GetSafeHwnd())
	{
		rtButton.left = 0;
		rtButton.right = rtButton.left + m_btnTabFavorite.GetWidth();
		rtButton.top = rtSearch.bottom;
		rtButton.bottom = rtButton.top + m_btnTabFavorite.GetHeight();

		m_btnTabFavorite.MoveWindow(rtButton, TRUE);
	}

	if (m_btnTabChatList.GetSafeHwnd())
	{
		rtButton.left = rtButton.right;
		rtButton.right = rtButton.left + m_btnTabChatList.GetWidth();
		rtButton.top = rtSearch.bottom;
		rtButton.bottom = rtButton.top + m_btnTabChatList.GetHeight();

		m_btnTabChatList.MoveWindow(rtButton, TRUE);
	}

}

void CMainDlg::AddChatRoomList(BOOL bGroup, int nRoomNo, LPCTSTR usersName, CTime time)
{
	if (m_bSearch)
	{
		CString searchText = m_wndSearch.GetSearchText();
		CString strUserName(usersName);

		if (-1 == strUserName.Find(searchText))
			return;
	}

	CSingleLock dialogLock(&m_csChatDialogList);
	dialogLock.Lock();

	// ̹ ߰Ǿ ִ ȭ ˻
	if (FindChatRoomIndex(nRoomNo) != -1)
	{
		dialogLock.Unlock();
		return;
	}

	int insertIndex = m_listChatRoom.GetItemCount();

	LVITEM lvItem;
	lvItem.mask = LVIF_IMAGE;
	lvItem.iItem = insertIndex;
	lvItem.iSubItem = 0;
	lvItem.iImage = 0;

	m_listChatRoom.InsertItem(&lvItem);
	dialogLock.Unlock();

	m_listChatRoom.SetItemData(insertIndex, nRoomNo);

	UpdateChatRoomList(nRoomNo, bGroup, usersName, time);
}

void CMainDlg::UpdateChatRoomList(int nRoomNo, BOOL bGroup, CString strUserName, CTime time)
{
	int nRoomIndex = FindChatRoomIndex(nRoomNo);

	if (nRoomIndex == -1)
		return;

	// ׷ ȭ  1:1 ȭ ǥ
	if (bGroup)
		m_listChatRoom.SetItemImage(nRoomIndex, COLUMN_INDEX_ICON, 1);
	else
		m_listChatRoom.SetItemImage(nRoomIndex, COLUMN_INDEX_ICON, 0);

	CString strTime = time.FormatGmt(_T("%m.%d  %H:%M"));

	if (!strUserName.IsEmpty())
		m_listChatRoom.SetItemText(nRoomIndex, COLUMN_INDEX_NAME, strUserName);

	if (!strTime.IsEmpty())
		m_listChatRoom.SetItemText(nRoomIndex, COLUMN_INDEX_LAST_MSG_TIME, strTime);
	
	m_listChatRoom.RedrawItems(nRoomIndex, nRoomIndex);
}

// äù ID  Ʈ Index  ٲش.
int CMainDlg::FindChatRoomIndex(int nRoomNo)
{
	int nRoomIndex = -1;

	CSingleLock dialogLock(&m_csChatDialogList);
	dialogLock.Lock();

	int nRoomCount = m_listChatRoom.GetItemCount();

	for (int i = 0; i < nRoomCount; i++)
	{
		int nItemRoomNo = (int)m_listChatRoom.GetItemData(i);

		if (nItemRoomNo == nRoomNo)
		{
			nRoomIndex = i;
			break;
		}
	}
	dialogLock.Unlock();

	return nRoomIndex;
}

void CMainDlg::ReleaseChatRoomList()
{
	if (NULL == m_listChatRoom.GetSafeHwnd())
		return;

	GET_APP()->m_pChatDialogManager->InitializeSearchRect();
	m_listChatRoom.DeleteAllItems();
}

void CMainDlg::RemoveChatRoom(CDialog* pDialog)
{
// 	ClearChatRoomList();
// 	GET_APP()->m_pChatDialogManager->InitChatRoomList();
	if (NULL == GetSafeHwnd())
		return;

	if (NULL == pDialog)
		return;

	if (FALSE == pDialog->IsKindOf(RUNTIME_CLASS(CChatDlg)))
		return;

	CSingleLock dialogLock(&m_csChatDialogList);
	dialogLock.Lock();

	int nCount = m_listChatRoom.GetItemCount();
	if (nCount <= 0)
	{
		dialogLock.Unlock();
		return;
	}

	CChatDlg *pDlg = (CChatDlg*)pDialog;
	if (NULL == pDlg)
	{
		dialogLock.Unlock();
		return;
	}

	for (int i = 0; i < nCount; ++i)
	{
		int nItemRoomNo = (int)m_listChatRoom.GetItemData(i);
		if (pDlg->GetRoomID() == nItemRoomNo)
		{
			//Ʈ 
			m_listChatRoom.EnsureVisible(0, TRUE);
			m_listChatRoom.DeleteItem(i);
			break;
		}
	}
	dialogLock.Unlock();
}

void CMainDlg::OnLvnItemchangedMainChatManager(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);

	if (!(pNMLV->uChanged == LVIF_STATE && pNMLV->uNewState == (LVIS_SELECTED | LVIS_FOCUSED)))
		return;

	if (m_bCloseDialog)
	{
		return;
	}

/*
	int nSelectedCount = m_listChatRoom.GetSelectedCount();

	if (1 < nSelectedCount)
	{
		AlignMeultiChatDialog(nSelectedCount);
	}
*/

	*pResult = 0;
}

void CMainDlg::OnNMClickMainChatManager(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMITEMACTIVATE pNMItemActivate = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);

	if (NULL == pNMItemActivate || m_bCloseDialog)
	{
		return;
	}

	if (pNMItemActivate->iItem < 0)
		return;

	POSITION pos = m_listChatRoom.GetFirstSelectedItemPosition();

	while (pos)
	{
		int nItem = m_listChatRoom.GetNextSelectedItem(pos);
		int nRoomNo = (int)m_listChatRoom.GetItemData(nItem);
		GET_APP()->m_pChatDialogManager->SetForegroundChatDialog(nRoomNo);
	}

	*pResult = 0;
}

void CMainDlg::OnBnClickedTileButton()
{
	GET_APP()->m_pChatDialogManager->InitializeScreenRects();

	CStringArray strRoomIDArray;
	BOOL bResize = FALSE;
	for (int i = 0 ;i < m_listChatRoom.GetItemCount() ; i++)
	{
		int nRoomNo = (int)m_listChatRoom.GetItemData(i);
		if (FALSE == GET_APP()->m_pChatDialogManager->ActivateChatDialog(nRoomNo, bResize))
		{
			// â ũ ġ  Ұ  â ũ⸦ ⺻ũ ϰ  ̵  ġ
			GET_MESSENGER_DLG()->SetWindowPos(NULL, 0, 0, DEF_MIN_SIZE_WIDTH, DEF_MIN_SIZE_HEIGHT, 0);
			GET_APP()->m_pChatDialogManager->InitializeScreenRects(TRUE);
			bResize = TRUE;
			i = -1;
		}
	}
}

void CMainDlg::ClearChatRoomList()
{
	if (NULL == GetSafeHwnd())
		return;

	CSingleLock dialogLock(&m_csChatDialogList);
	dialogLock.Lock();
	ReleaseChatRoomList();
	dialogLock.Unlock();

	m_wndSearch.SetIntroducingText();
	m_wndSearch.SetSearchText(_T(""));
}

void CMainDlg::OnBnClickedTabFavorite()
{
	m_btnTabFavorite.SetTapSelected();
	m_btnTabChatList.SetTapUnSelected();

	if (NULL != m_favoriteController.GetTreeCtrl().GetSafeHwnd())
		m_favoriteController.GetTreeCtrl().ShowWindow(SW_SHOW);
	m_listChatRoom.ShowWindow(SW_HIDE);

	m_wndSearch.SetSearchChatRoom(FALSE);
	m_wndSearch.SetDefaultText(LoadStringFromResource(IDS_STRING245));
	m_wndSearch.SetIntroducingText();

	if (m_bSearch)
	{
		m_wndSearch.SetSearchText(_T(""));
	}
	m_wndSearch.Invalidate();
}

void CMainDlg::OnBnClickedTabChatList()
{
	m_btnTabFavorite.SetTapUnSelected();
	m_btnTabChatList.SetTapSelected();

	if (NULL != m_favoriteController.GetTreeCtrl().GetSafeHwnd())
		m_favoriteController.GetTreeCtrl().ShowWindow(SW_HIDE);
	m_listChatRoom.ShowWindow(SW_SHOW);

	m_wndSearch.SetSearchChatRoom(TRUE);
	m_wndSearch.SetDefaultText(LoadStringFromResource(IDS_STRING352));
	m_wndSearch.SetIntroducingText();

	if (m_bSearch && FALSE == m_strOpenChatSearchext.IsEmpty())
	{
		InvalidateChatListSearch();
	}
	m_wndSearch.Invalidate();
}

BOOL CMainDlg::OnEraseBkgnd(CDC* pDC)
{
	//pdc null 찡 ִ-_-;
	// null ?
	if (pDC == NULL)
		return TRUE;
	CRect rcClient;
	GetClientRect(&rcClient);

	NMMemDC memDC;
	memDC.CreateCompatibleDC(pDC, &rcClient);
	memDC.FillSolidRect(0, 0, rcClient.Width(), rcClient.Height(), RGB(255, 255, 255));	// ȭ 

	//  ̹
	BITMAP bmTopBgInfo, bmSearchBgInfo;
	NMMemDC memBmpDC;

	// Ⱓ ý ư 
	m_bmLegacyBg.GetBitmap(&bmTopBgInfo);
	memBmpDC.CreateCompatibleDC(pDC, bmTopBgInfo.bmWidth, bmTopBgInfo.bmHeight);

	int nX = 0;
	int nY = 0;

	if (m_wndInfoAboutMe.GetSafeHwnd())
	{
		CRect rt;
		m_wndInfoAboutMe.GetWindowRect(&rt);
		ScreenToClient(&rt);

		nY = rt.bottom;
	}

	CBitmap *pOldBmp = memBmpDC.SelectObject(&m_bmLegacyBg);
	memDC.StretchBlt(nX, nY, rcClient.Width(), bmTopBgInfo.bmHeight, &memBmpDC, 0, 0, bmTopBgInfo.bmWidth, bmTopBgInfo.bmHeight, SRCCOPY);
	memBmpDC.SelectObject(pOldBmp);
	memBmpDC.DeleteDC();

	// ˻ â 
	m_bmSearchBg.GetBitmap(&bmSearchBgInfo);
	memBmpDC.CreateCompatibleDC(pDC, bmSearchBgInfo.bmWidth, bmSearchBgInfo.bmHeight);

	nY += DEF_LEGACY_BTN_HEIGHT;

	pOldBmp = memBmpDC.SelectObject(&m_bmSearchBg);
	memDC.StretchBlt(nX, nY, rcClient.Width(), bmSearchBgInfo.bmHeight, &memBmpDC, 0, 0, bmSearchBgInfo.bmWidth, bmSearchBgInfo.bmHeight, SRCCOPY);
	memBmpDC.SelectObject(pOldBmp);
	memBmpDC.DeleteDC();

	if (m_wndSearch.GetSafeHwnd())
	{
		CRect rcSearch;
		m_wndSearch.GetWindowRect(rcSearch);
		ScreenToClient(rcSearch);
		memDC.FillSolidRect(rcSearch.left - 1, rcSearch.top, 1, rcSearch.Height() - 1, RGB(190,195,205));
	}

	// ˻â Ʒ  ư  
	m_bmTopBg.GetBitmap(&bmTopBgInfo);
	memBmpDC.CreateCompatibleDC(pDC, bmTopBgInfo.bmWidth, bmTopBgInfo.bmHeight);

	nX = 0;
	nY = 0;

	if (m_btnTabFavorite.GetSafeHwnd())
	{
		CRect rtTabButton;
		m_btnTabFavorite.GetWindowRect(&rtTabButton);
		ScreenToClient(&rtTabButton);

		nX = m_btnTabFavorite.GetWidth() * 2;
		nY = rtTabButton.top;
	}

	pOldBmp = memBmpDC.SelectObject(&m_bmTopBg);
	memDC.StretchBlt(nX, nY, rcClient.Width(), bmTopBgInfo.bmHeight, &memBmpDC, 0, 0, bmTopBgInfo.bmWidth, bmTopBgInfo.bmHeight, SRCCOPY);

	memBmpDC.SelectObject(pOldBmp);
	memBmpDC.DeleteDC();

	// memDC  ׸  ѹ DC ׷  .
	pDC->BitBlt(0, 0, rcClient.Width(), rcClient.Height(), &memDC, 0, 0, SRCCOPY);	
	memDC.DeleteDC();

	return TRUE;
}

void CMainDlg::InvalidateChatListSearch()
{
	m_wndSearch.SetIntroducingText(m_strOpenChatSearchext);
	m_wndSearch.SetSearchText(m_strOpenChatSearchext);
	m_wndSearch.Invalidate();
}

void CMainDlg::AlignMeultiChatDialog( int nSelectedCount )
{
	GET_APP()->m_pChatDialogManager->InitializeScreenRects();

	POSITION pos = m_listChatRoom.GetFirstSelectedItemPosition();
	BOOL bResize = FALSE;

	while (pos)
	{
		int nItem = m_listChatRoom.GetNextSelectedItem(pos);
		int nRoomNo = (int)m_listChatRoom.GetItemData(nItem);

		if (FALSE == GET_APP()->m_pChatDialogManager->ActivateChatDialog(nRoomNo, bResize))
		{
			// â ũ ġ  Ұ  â ũ⸦ ⺻ũ ϰ  ̵  ġ
			GET_MESSENGER_DLG()->SetWindowPos(NULL, 0, 0, DEF_MIN_SIZE_WIDTH, DEF_MIN_SIZE_HEIGHT, 0);
			GET_APP()->m_pChatDialogManager->InitializeScreenRects(TRUE);
			bResize = TRUE;
			
			break;
		}
	}
}

void CMainDlg::OnMenuChatManager(WPARAM wParam)
{
	int nID = (int)wParam;
	TRACE(_T("OnMenuChatManager = %d \n"), nID);

	int nSelectedCount = m_listChatRoom.GetSelectedCount();

	if (nSelectedCount <= 0)
		return;

	CUIntArray nRoomIDArray;
	POSITION pos = m_listChatRoom.GetFirstSelectedItemPosition();

	while (pos)
	{
		int nItem = m_listChatRoom.GetNextSelectedItem(pos);
		int nRoomNo = (int)m_listChatRoom.GetItemData(nItem);
		nRoomIDArray.Add(nRoomNo);
	}

	switch (nID)
	{
	case ID_CHAT_ROOM_OPEN:
		{
			POSITION pos = m_listChatRoom.GetFirstSelectedItemPosition();

			while (pos)
			{
				int nItem = m_listChatRoom.GetNextSelectedItem(pos);
				int nRoomNo = (int)m_listChatRoom.GetItemData(nItem);
				GET_APP()->m_pChatDialogManager->SetForegroundChatDialog(nRoomNo);
			}
		}
		break;

	case ID_CHAT_ROOM_CLOSE:
		if (NMessageBox(this, LoadStringFromResource(IDS_STRING348), _T("Ntalk biz"), MB_YESNO) == IDYES)
		{
			m_bCloseDialog = TRUE;
			GET_APP()->m_pChatDialogManager->CloseChatDialogs(nRoomIDArray);
			m_bCloseDialog = FALSE;
		}
		break;

	case ID_CHAT_ROOM_ALL_CLOSE:
		if (NMessageBox(this, LoadStringFromResource(IDS_STRING349), _T("Ntalk biz"), MB_YESNO) == IDYES)
		{
			m_bCloseDialog = TRUE;
			GET_APP()->m_pChatDialogManager->CloseAllChatDialog();
			m_bCloseDialog = FALSE;
		}
		break;
	}
}

void CMainDlg::OnMenuFavoriteAdd(WPARAM wParam)
{
	int nID = (int)wParam;
	GROUP_MEMU_LIST::iterator iter = m_GroupList.find(nID);
	int nGroupID = iter->second;

	POSITION pos = m_listChatRoom.GetFirstSelectedItemPosition();
	if (pos == NULL)
	{
		NMessageBox(this, LoadStringFromResource(IDS_STRING218));
		return;
	}

	CStringList selectedIdList;
	GetSelectedUserIDList(selectedIdList);	// õ ID Ʈ

	std::vector<wstring> vecBuddyId;

	for (pos = selectedIdList.GetHeadPosition(); pos != NULL; )
	{
		vecBuddyId.push_back((LPCWSTR)selectedIdList.GetNext(pos));
	}
	
	if (selectedIdList.GetCount() > 0)
		g_pMessengerActor->RequestAddBuddy(nGroupID, vecBuddyId, _T(""));
}

void CMainDlg::ShowAlignHelpMsg()
{
	if (GET_CONFIG()->GetChatAlignHelpMsgShowingCount() >= 5)
		return;

	CPoint ptStart(0, 0);

	if (!m_btnTile.GetSafeHwnd())
		return;
	
	CRect rtButton;
	m_btnTile.GetWindowRect(&rtButton);

	ptStart.x = rtButton.left - 138;
	ptStart.y = rtButton.bottom;

	m_pChatAlignHelpLayerDlg = new CLayeredDlg();
	m_pChatAlignHelpLayerDlg->SetImgBGPath(_T("Main_bg_layer.png"));
	m_pChatAlignHelpLayerDlg->SetTitleAndMessage(LoadStringFromResource(IDS_STRING365), LoadStringFromResource(IDS_STRING366));
	m_pChatAlignHelpLayerDlg->Create(CLayeredDlg::IDD, CWnd::GetDesktopWindow());
	m_pChatAlignHelpLayerDlg->SetWindowPos(&CWnd::wndTop, ptStart.x, ptStart.y, 0, 0, SWP_NOSIZE | SWP_SHOWWINDOW);
}

void CMainDlg::__ResourceLoad(BOOL bInit)
{
	if (bInit == TRUE)
	{
		if (m_btnTile.GetSafeHwnd())
		{
			m_btnTile.SetBtnBitmapFile(GetAppImgFile(_T("Main_align_tile.bmp")), RGB(0,0,0));
			m_btnTile.SetToolTipText(LoadStringFromResource(IDS_STRING351));
		}

		if (m_btnLegacyPre.GetSafeHwnd() && m_btnLegacyPost.GetSafeHwnd())
		{
			m_btnLegacyPre.SetBtnBitmapFile(GetAppImgFile(_T("btn_scroll_left.bmp")), RGB(0,0,0));
			m_btnLegacyPost.SetBtnBitmapFile(GetAppImgFile(_T("btn_scroll_right.bmp")), RGB(0,0,0));
		}

		if (m_bEnableNotice && m_staticNotice.GetSafeHwnd())
			m_staticNotice.SetPrefixString(LoadStringFromResource(IDS_STRING_MAIN_NOTICE));	

		if (m_btnTabChatList.GetSafeHwnd())
		{
			m_btnTabChatList.SetTabBitmapFile(GetAppImgFile(_T("Main_tab_02.bmp")),RGB(0,0,0));
			m_btnTabChatList.SetFont(&g_ftDefault9PT);
			m_btnTabChatList.SetTextColor(RGB(78, 157, 199), RGB(34, 34, 34));
			m_btnTabChatList.SetButtonText(LoadStringFromResource(IDS_STRING_CHAT_ATTENDING_LIST));
			m_btnTabChatList.ShowWindow(SW_SHOW);
		}

		if (m_btnTabFavorite.GetSafeHwnd())
		{
			m_btnTabFavorite.SetTabBitmapFile(GetAppImgFile(_T("Main_tab_01.bmp")),RGB(0,0,0));
			m_btnTabFavorite.SetFont(&g_ftDefault9PT);
			m_btnTabFavorite.SetTextColor(RGB(78, 157, 199), RGB(34, 34, 34));
			m_btnTabFavorite.SetButtonText(LoadStringFromResource(IDS_STRING_CHAT_LIST));
			m_btnTabFavorite.ShowWindow(SW_SHOW);
		}
	}
	else
	{
		if (m_btnTile.GetSafeHwnd())
		{
			m_btnTile.SetToolTipText(LoadStringFromResource(IDS_STRING351));
		}

		if (m_bEnableNotice && m_staticNotice.GetSafeHwnd())
			m_staticNotice.SetPrefixString(LoadStringFromResource(IDS_STRING_MAIN_NOTICE));	

		if (m_btnTabChatList.GetSafeHwnd())
		{
			m_btnTabChatList.SetButtonText(LoadStringFromResource(IDS_STRING_CHAT_ATTENDING_LIST));
		}

		if (m_btnTabFavorite.GetSafeHwnd())
		{
			m_btnTabFavorite.SetButtonText(LoadStringFromResource(IDS_STRING_CHAT_LIST));
		}
	}

	if(g_stNtalkInfo.nLang == NTALK_LANG_KOREAN) //kor
	{

	}
	else if(g_stNtalkInfo.nLang == NTALK_LANG_ENGLISH) //eng
	{


	}
	else if(g_stNtalkInfo.nLang == NTALK_LANG_JAPANESE)
	{

	}
	else if(g_stNtalkInfo.nLang == NTALK_LANG_CHINESE_TRADITIONAL)
	{

	}
	else if(g_stNtalkInfo.nLang == NTALK_LANG_CHINESE_SIMPLIFIED)
	{

	}

	if (bInit == FALSE)
		SendMessage(WM_SIZE, NULL, NULL);
}

void CMainDlg::ImportGroup()
{
	m_favoriteController.ImportGroup();
}

LONG CMainDlg::ResizeLegacyButtons(CRect &rcClient, CRect &rtAboutMe)
{
	CRect rtLegacyItem;
	rtLegacyItem.SetRectEmpty();
	int nLegacyBtnBgWidth = rcClient.Width();	// Ⱓ ư   ũ

	if (nLegacyBtnBgWidth <= 0)
		return 0L;

	if (!m_btnLegacyPre.GetSafeHwnd() ||
		!m_btnLegacyPost.GetSafeHwnd() ||
		!m_wndInfoItemChat.GetSafeHwnd() ||
		!m_wndInfoItemNote.GetSafeHwnd() ||
		!m_wndInfoItemSign.GetSafeHwnd() ||
		!m_wndInfoItemMail.GetSafeHwnd() ||
		!m_wndInfoItemVOC.GetSafeHwnd() ||
		!m_wndInfoItemSurvey.GetSafeHwnd() ||
		!m_wndInfoItemIN.GetSafeHwnd() ||
		!m_wndInfoItemSAM.GetSafeHwnd() ||
		!m_wndInfoItemWorkOrg.GetSafeHwnd())
	return 0L;

	// Ⱓ  ũ ư
	rtLegacyItem.left = 0;
	rtLegacyItem.right = rtLegacyItem.left + m_btnLegacyPre.GetImgWidth();
	rtLegacyItem.top = rtAboutMe.bottom;
	rtLegacyItem.bottom = rtLegacyItem.top + DEF_LEGACY_BTN_HEIGHT;

	nLegacyBtnBgWidth -= rtLegacyItem.Width();

	m_btnLegacyPre.MoveWindow(&rtLegacyItem, FALSE);

	// Ⱓ  ũ ư
	rtLegacyItem.right = rcClient.right;
	rtLegacyItem.left = rtLegacyItem.right - m_btnLegacyPost.GetImgWidth();
	rtLegacyItem.top = rtAboutMe.bottom;
	rtLegacyItem.bottom = rtLegacyItem.top + DEF_LEGACY_BTN_HEIGHT;

	nLegacyBtnBgWidth -= rtLegacyItem.Width();

	m_btnLegacyPost.MoveWindow(&rtLegacyItem, FALSE);

	int nEnableLegacyCount = 0;
	bool bAlignLeft = false;

	// Ⱓ  ư ѿ   ڰ ٲ  ڿ  ư 迩ΰ ǹǷ,   ư  
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoChat)		nEnableLegacyCount++;
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoNote)		nEnableLegacyCount++;
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoMail)		nEnableLegacyCount++;
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSign)		nEnableLegacyCount++;
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoVOC)		nEnableLegacyCount++;
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSurvay)	nEnableLegacyCount++;
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoIN)		nEnableLegacyCount++;
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSAM)		nEnableLegacyCount++;
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoWorkOrg)	nEnableLegacyCount++;

	if ((nEnableLegacyCount * DEF_LEGACY_BTN_WIDTH) <= nLegacyBtnBgWidth)
	{
		m_btnLegacyPre.EnableWindow(FALSE);
		m_btnLegacyPost.EnableWindow(FALSE);
		
		m_nLegacyBtnSX = 0;
		bAlignLeft = true;
	}
	else
	{
		m_btnLegacyPre.EnableWindow(TRUE);
		m_btnLegacyPost.EnableWindow(TRUE);
	}

	if (!bAlignLeft && (m_nLegacyBtnMoveSizeRight == DEF_LEGACY_BTN_WIDTH) && IsRemainLeftLegacyItem())
	{
		LONG nSRX = nLegacyBtnBgWidth + m_btnLegacyPre.GetImgWidth();
		m_bRightLegacyItemShown = false;

		MoveLegacyButtonReverse(m_wndInfoItemWorkOrg, GET_CONFIG()->GetLegacyBtnInfo().bInfoWorkOrg, nSRX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButtonReverse(m_wndInfoItemSAM, GET_CONFIG()->GetLegacyBtnInfo().bInfoSAM, nSRX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButtonReverse(m_wndInfoItemIN, GET_CONFIG()->GetLegacyBtnInfo().bInfoIN, nSRX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButtonReverse(m_wndInfoItemSurvey, GET_CONFIG()->GetLegacyBtnInfo().bInfoSurvay, nSRX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButtonReverse(m_wndInfoItemVOC, GET_CONFIG()->GetLegacyBtnInfo().bInfoVOC, nSRX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButtonReverse(m_wndInfoItemSign, GET_CONFIG()->GetLegacyBtnInfo().bInfoSign, nSRX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButtonReverse(m_wndInfoItemMail, GET_CONFIG()->GetLegacyBtnInfo().bInfoMail, nSRX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButtonReverse(m_wndInfoItemNote, GET_CONFIG()->GetLegacyBtnInfo().bInfoNote, nSRX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButtonReverse(m_wndInfoItemChat, GET_CONFIG()->GetLegacyBtnInfo().bInfoChat, nSRX, rtAboutMe.bottom, nLegacyBtnBgWidth);

		m_nLegacyBtnGap = nLegacyBtnBgWidth;
		m_nLegacyBtnMoveSizeLeft = abs(DEF_LEGACY_BTN_WIDTH - nLegacyBtnBgWidth);
		m_nLegacyBtnMoveSizeRight = DEF_LEGACY_BTN_WIDTH;

		if (m_nOldLegacyBgWidth != rcClient.Width())
		{
			m_nLegacyBtnSX = 0;
		}
	}
	else
	{
		LONG nSX = m_nLegacyBtnSX + m_btnLegacyPre.GetImgWidth();

		MoveLegacyButton(m_wndInfoItemChat, GET_CONFIG()->GetLegacyBtnInfo().bInfoChat, nSX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButton(m_wndInfoItemNote, GET_CONFIG()->GetLegacyBtnInfo().bInfoNote, nSX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButton(m_wndInfoItemMail, GET_CONFIG()->GetLegacyBtnInfo().bInfoMail, nSX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButton(m_wndInfoItemSign, GET_CONFIG()->GetLegacyBtnInfo().bInfoSign, nSX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButton(m_wndInfoItemVOC, GET_CONFIG()->GetLegacyBtnInfo().bInfoVOC, nSX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButton(m_wndInfoItemSurvey, GET_CONFIG()->GetLegacyBtnInfo().bInfoSurvay, nSX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButton(m_wndInfoItemIN, GET_CONFIG()->GetLegacyBtnInfo().bInfoIN, nSX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButton(m_wndInfoItemSAM, GET_CONFIG()->GetLegacyBtnInfo().bInfoSAM, nSX, rtAboutMe.bottom, nLegacyBtnBgWidth);
		MoveLegacyButton(m_wndInfoItemWorkOrg, GET_CONFIG()->GetLegacyBtnInfo().bInfoWorkOrg, nSX, rtAboutMe.bottom, nLegacyBtnBgWidth);

		m_nLegacyBtnGap = nLegacyBtnBgWidth;
		m_nLegacyBtnMoveSizeRight = abs(DEF_LEGACY_BTN_WIDTH - nLegacyBtnBgWidth);
		m_nLegacyBtnMoveSizeLeft = DEF_LEGACY_BTN_WIDTH;
	}

	m_nOldLegacyBgWidth = rcClient.Width();

	return rtLegacyItem.bottom;
}

void CMainDlg::MoveLegacyButton(CInfoItemWnd &infoItem, BOOL bEnable, LONG &nSX, LONG nSY, int &nLegacyBtnBgWidth)
{
	if (bEnable && nLegacyBtnBgWidth >= DEF_LEGACY_BTN_WIDTH)
	{
		CRect rtLegacyItem;
		rtLegacyItem.left = nSX;
		rtLegacyItem.right = rtLegacyItem.left + DEF_LEGACY_BTN_WIDTH;
		rtLegacyItem.top = nSY;
		rtLegacyItem.bottom = rtLegacyItem.top + infoItem.GetImgBGHeight();

		if (nSX >= m_btnLegacyPre.GetImgWidth())
		{
			nLegacyBtnBgWidth -= DEF_LEGACY_BTN_WIDTH;

			infoItem.MoveWindow(&rtLegacyItem, TRUE);
			infoItem.ShowWindow(SW_SHOW);
		}
		else
		{
			infoItem.ShowWindow(SW_HIDE);

			if (rtLegacyItem.right >= m_btnLegacyPre.GetImgWidth())
				nLegacyBtnBgWidth -= rtLegacyItem.right - m_btnLegacyPre.GetImgWidth();
		}

		nSX += DEF_LEGACY_BTN_WIDTH;
	}
	else
	{
		infoItem.ShowWindow(SW_HIDE);
	}
}

void CMainDlg::MoveLegacyButtonReverse(CInfoItemWnd &infoItem, BOOL bEnable, LONG &nSX, LONG nSY, int &nLegacyBtnBgWidth)
{
	if (!m_bRightLegacyItemShown && 
		!infoItem.IsWindowVisible())
		return;

	if (bEnable && nLegacyBtnBgWidth >= DEF_LEGACY_BTN_WIDTH)
	{
		CRect rtLegacyItem;
		rtLegacyItem.right = nSX;
		rtLegacyItem.left = rtLegacyItem.right - DEF_LEGACY_BTN_WIDTH;
		rtLegacyItem.top = nSY;
		rtLegacyItem.bottom = rtLegacyItem.top + infoItem.GetImgBGHeight();

		if (rtLegacyItem.left >= m_btnLegacyPre.GetImgWidth())
		{
			nLegacyBtnBgWidth -= DEF_LEGACY_BTN_WIDTH;

			infoItem.MoveWindow(&rtLegacyItem, TRUE);
			infoItem.ShowWindow(SW_SHOW);
			m_bRightLegacyItemShown = true;
		}
		else
		{
			infoItem.ShowWindow(SW_HIDE);

			// 				if (rtLegacyItem.right >= m_btnLegacyPre.GetImgWidth())
			// 					nLegacyBtnBgWidth -= rtLegacyItem.right - m_btnLegacyPre.GetImgWidth();
		}

		nSX -= DEF_LEGACY_BTN_WIDTH;
	}
	else
	{
		infoItem.ShowWindow(SW_HIDE);
	}
}

void CMainDlg::OnBnClickedButtonMainLegacyPre()
{
	CRect rcClient;
	GetClientRect(rcClient);
	LONG nSY = rcClient.top + m_wndInfoAboutMe.GetImgBGHeight();

	m_nLegacyBtnSX += m_nLegacyBtnMoveSizeLeft;
	m_nLegacyBtnMoveSizeLeft = DEF_LEGACY_BTN_WIDTH;
	m_nLegacyBtnMoveSizeRight = abs(DEF_LEGACY_BTN_WIDTH - m_nLegacyBtnGap);

	if (m_nLegacyBtnSX >= 0)
		m_nLegacyBtnSX = 0;

	LONG nSX = m_nLegacyBtnSX + m_btnLegacyPre.GetImgWidth();
	int nLegacyBtnBgWidth = rcClient.Width() - m_btnLegacyPre.GetImgWidth() - m_btnLegacyPost.GetImgWidth();

	MoveLegacyButton(m_wndInfoItemChat, GET_CONFIG()->GetLegacyBtnInfo().bInfoChat, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemNote, GET_CONFIG()->GetLegacyBtnInfo().bInfoNote, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemMail, GET_CONFIG()->GetLegacyBtnInfo().bInfoMail, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemSign, GET_CONFIG()->GetLegacyBtnInfo().bInfoSign, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemVOC, GET_CONFIG()->GetLegacyBtnInfo().bInfoVOC, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemSurvey, GET_CONFIG()->GetLegacyBtnInfo().bInfoSurvay, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemIN, GET_CONFIG()->GetLegacyBtnInfo().bInfoIN, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemSAM, GET_CONFIG()->GetLegacyBtnInfo().bInfoSAM, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemWorkOrg, GET_CONFIG()->GetLegacyBtnInfo().bInfoWorkOrg, nSX, nSY, nLegacyBtnBgWidth);
}

void CMainDlg::OnBnClickedButtonMainLegacyPost()
{
	CRect rcClient;
	GetClientRect(rcClient);
	LONG nSY = rcClient.top + m_wndInfoAboutMe.GetImgBGHeight();

	// 鿡 ư ϳ ̻  (, ũ  )
	if (m_nLegacyBtnGap >= DEF_LEGACY_BTN_WIDTH)
		return;

	// ʿ   Ⱓ ư ִ ˻Ѵ.
	if (!IsRemainRightLegacyItem())
		return;

	m_nLegacyBtnSX -= m_nLegacyBtnMoveSizeRight;
	m_nLegacyBtnMoveSizeRight = DEF_LEGACY_BTN_WIDTH;
	m_nLegacyBtnMoveSizeLeft = abs(DEF_LEGACY_BTN_WIDTH - m_nLegacyBtnGap);

	LONG nSX = m_nLegacyBtnSX + m_btnLegacyPre.GetImgWidth();
	int nLegacyBtnBgWidth = rcClient.Width() - m_btnLegacyPre.GetImgWidth() - m_btnLegacyPost.GetImgWidth();

	MoveLegacyButton(m_wndInfoItemChat, GET_CONFIG()->GetLegacyBtnInfo().bInfoChat, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemNote, GET_CONFIG()->GetLegacyBtnInfo().bInfoNote, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemMail, GET_CONFIG()->GetLegacyBtnInfo().bInfoMail, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemSign, GET_CONFIG()->GetLegacyBtnInfo().bInfoSign, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemVOC, GET_CONFIG()->GetLegacyBtnInfo().bInfoVOC, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemSurvey, GET_CONFIG()->GetLegacyBtnInfo().bInfoSurvay, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemIN, GET_CONFIG()->GetLegacyBtnInfo().bInfoIN, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemSAM, GET_CONFIG()->GetLegacyBtnInfo().bInfoSAM, nSX, nSY, nLegacyBtnBgWidth);
	MoveLegacyButton(m_wndInfoItemWorkOrg, GET_CONFIG()->GetLegacyBtnInfo().bInfoWorkOrg, nSX, nSY, nLegacyBtnBgWidth);
}

bool CMainDlg::IsRemainRightLegacyItem()
{
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoWorkOrg)
	{
		if (m_wndInfoItemWorkOrg.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSAM)
	{
		if (m_wndInfoItemSAM.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoIN)
	{
		if (m_wndInfoItemIN.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSurvay)
	{
		if (m_wndInfoItemSurvey.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoVOC)
	{
		if (m_wndInfoItemVOC.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSign)
	{
		if (m_wndInfoItemSign.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoMail)
	{
		if (m_wndInfoItemMail.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoNote)
	{
		if (m_wndInfoItemNote.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoChat)
	{
		if (m_wndInfoItemChat.IsWindowVisible())
			return false;
		else
			return true;
	}

	return false;
}

bool CMainDlg::IsRemainLeftLegacyItem()
{
	if (GET_CONFIG()->GetLegacyBtnInfo().bInfoChat)
	{
		if (m_wndInfoItemChat.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoNote)
	{
		if (m_wndInfoItemNote.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoMail)
	{
		if (m_wndInfoItemMail.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSign)
	{
		if (m_wndInfoItemSign.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoVOC)
	{
		if (m_wndInfoItemVOC.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSurvay)
	{
		if (m_wndInfoItemSurvey.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoIN)
	{
		if (m_wndInfoItemIN.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoSAM)
	{
		if (m_wndInfoItemSAM.IsWindowVisible())
			return false;
		else
			return true;
	}
	else if (GET_CONFIG()->GetLegacyBtnInfo().bInfoWorkOrg)
	{
		if (m_wndInfoItemWorkOrg.IsWindowVisible())
			return false;
		else
			return true;
	}

	return false;
}

void CMainDlg::InitLegacyBtnPos()
{
	m_nLegacyBtnGap = 0;
	m_nLegacyBtnSX = 0;
	m_nLegacyBtnMoveSizeRight = 0;
	m_nLegacyBtnMoveSizeLeft = 0;
	m_bRightLegacyItemShown = false;
	m_nOldLegacyBgWidth = 0;
}
